import json
import time
import os
import csv
import nltk

article_keywords = [
    'politik',
    'parti',

    'høyre',
    '(h)',
    'venstre',
    '(v)',
    # 'arbeiderpartiet',
    '(ap)',
    '(a)',
    # 'fremskrittspartiet',
    'frp',
    # 'kristelig folkeparti',
    'krf',
    # 'miljøpartiet de grønne',
    'mdg',
    # 'rødt',
    '(r)',
    # 'senterpartiet',
    '(sp)',
    # 'sosialistisk venstreparti',
    '(sv)',

    'demokratene i norge',
    '(din)',
    # 'kystpartiet',
    '(kp)',
    # 'partiet de kristne',
    '(pdk)',
    # 'pensjonspartiet',
    '(pp)',

    'ordfører',
    'minister',
    'leder',
    'rådmann',

    # 'styre',

    'regjering',
    'storting',
    'fylkesting',

    # 'kommune',
    # 'fylke',
    'staten',
]


def write_file(content, filename):
    with open(filename, 'w', encoding='utf-8') as myfile:
        writer = csv.writer(myfile, delimiter=',', lineterminator='\n')
        for key, value in content.items():
            writer.writerow([key, value[0], value[1]])


path = 'E:/contentdata'

data1 = path+'/content-20170207.data'
data2 = path+'/content-20170215.data'

any_words = [', sier', 'les også', 'saken oppdateres', 'interaktivt kart']


# Use nltk to tokenize text into sentences
def split_text_to_sentences(text):
    tokenizer = nltk.data.load('tokenizers/punkt/norwegian.pickle')
    tokens = tokenizer.tokenize(text)
    filtered_tokens = list(filter(lambda x: len(x.split()) > 5, tokens))  # Remove sentences shorter than 5 words
    return filtered_tokens


def get_relevant_articles_from_file(file, save_as_filename):
    start = time.clock()
    print("Getting relevant articles from file: ", file)
    rel_sentences = {}
    error_set = set()
    url_set = set()

    counter = 0
    id_counter = 0

    with open(file, 'r', encoding='utf-8') as infile:
        print("Loaded file")
        for line in infile:
            counter += 1

            if counter == 1:
                print("First counter!")

            if counter % 50000 == 0:
                print("Counter: ", counter)

            # if len(url_set) < 2:
            try:
                correct_category0 = False
                correct_category1 = False

                json_to_python = json.loads(line)
                # print(json.dumps(json_to_python, indent=4, sort_keys=True))  # Pretty printing json obj.

                body = None
                url = None
                description = None
                title = None

                for i in json_to_python['fields']:
                    field = i['field']
                    value = i['value']

                    if field == 'category0':
                        if value == 'nyheter':
                            correct_category0 = True

                    if correct_category0 and field == 'category1':
                        if value == 'nyheter|politikk' or value == 'nyheter|innenriks':
                            correct_category1 = True

                    if field == 'body':
                        body = value

                    if field == 'description':
                        description = value

                    if field == 'title':
                        title = value

                    if field == 'url':
                        url = value

                if correct_category1 and url is None:
                    print("No url")

                if correct_category1 and url not in url_set and 'utenriks' not in url.lower():
                    url_set.add(url)

                    if 'valgomat' not in title.lower() and 'quiz' not in title.lower():
                        for word in article_keywords:
                            if word.lower() in title.lower():
                                content = body
                                content.append(title)
                                content.append(description)

                                for elem in content:
                                    sentences = split_text_to_sentences(elem)
                                    for sentence in sentences:
                                        # Generate id for each item
                                        if any(word in sentence.lower() for word in any_words):
                                            continue
                                        else:
                                            uid = 'setn-{0:.3f}'.format(id_counter / 1000).replace('.', '')
                                            id_counter += 1
                                            rel_sentences[uid] = [sentence, url]
                                            # print("\t", uid, sentence, url)

            except TypeError as e:
                error_set.add(e)
            # else:
            #    break

        print("Relevant sentences: ")
        # for j in rel_sentences:
        #    print(j, rel_sentences[j])
        write_file(rel_sentences, save_as_filename)
        end = time.clock()
        print("Time: ", end - start)


def get_rel_art_from_files(files):
    count = 1
    for file in files:
        get_relevant_articles_from_file(file, ("rel_sentences_{}.csv".format(count)))
        count += 1


# get_relevant_articles_from_file(data2, ("rel_sentences_{}.csv".format(2)))
get_rel_art_from_files([data1, data2])
