from analyze.db import Database
import csv
from collections import Counter

db = Database('final_ccdb01.db')

'''
Hente ut alle setninger med svar 1 eller 2
Sjekke hvilke grupper som er enige med 1 eller 2 og hvilke som mener motsatt
'''

# all answers based on gender, education, age, score,
# where unlabelede_id in "All claims that have more than 3 answers"
# and not in "All claims where the answer = 0"
result = db.fetch('select a.unlabeled_id, a.answer, u.gender, u.education, u.age, u.score '
                  'from Answers as a INNER JOIN Users as u ON a.username = u.username '
                  'where a.unlabeled_id is not null AND a.answer != 0 '
                  'AND a.unlabeled_id in '
                  '( SELECT an.unlabeled_id '
                  'FROM Answers as an '
                  'WHERE an.unlabeled_id IS NOT NULL '
                  'GROUP BY an.unlabeled_id HAVING count(an.answer) > 3) '
                  'order by a.unlabeled_id')


claim_dict = {}

claims = db.fetch('SELECT * FROM UnlabeledClaims')
for row in claims:
    claim_dict[row[0]] = row[1]


gender_set = set()
education_set = set()
age_set = set()
score_set = set()

result_dict = {}

replacement_dict = {
    'kvinne': 'female',
    'mann': 'male',
    'annet': 'other',
    'humaniora': 'humanities',
    'idrett': 'sports_education',
    'kunst': 'art_and_design',
    'medisin': 'medicine_and_ontology',
    'realfag': 'natural_science',
    'rettsvitenskap': 'jurisprudence',
    'samfunnsvitenskap': 'social_sciences',
    'yrkesfag': 'vocational_education'
}


# Har allerede tatt vekk spørsmål som har svaret 0, så de som er igjen kan bare ha fått svarene 1,2 eller 3
def add_result_to_dict():
    for e in result:
        question_id, answer, gender, edu, age, score = e
        gender = replacement_dict[gender]
        edu = replacement_dict[edu]
        gender_set.add(gender)
        education_set.add(edu)
        age_set.add(age)
        score_set.add(score)
        question = claim_dict[question_id]
        if question_id not in result_dict.keys():
            result_dict[question_id] = [[answer, gender, edu, age, score, question]]
        else:
            result_dict[question_id].append([answer, gender, edu, age, score, question])


correct_answer_dict = {}


def add_correct_answers_to_dict(score_num):
    minus_one_count = 0
    with open('results/matrix_agreement_score{}.csv'.format(score_num), 'r', encoding='utf-8') as in_file:
        reader = csv.reader(in_file)
        header = next(reader)
        # print(header)
        for r in reader:
            qid, agreed_a, rate, q = r
            qid = int(float(qid))
            agreed_a = int(float(agreed_a))
            if agreed_a == -1:
                minus_one_count += 1
            elif qid not in correct_answer_dict:
                correct_answer_dict[qid] = {'answer': int(float(agreed_a))}
            else:
                print('Is in dict already!')
    print("-1 count for {}: {}".format(score_num, minus_one_count))


# agreed_answer = {}
# uncertain_answer = {}
included_answers = {}


def add_agreed_and_uncertain_to_dict():
    excluded_answers = 0
    for key, value in result_dict.items():
        if key not in correct_answer_dict:
            excluded_answers += 1
        else:
            included_answers[key] = (correct_answer_dict[key], value)
    print("Excluded answers: ", excluded_answers)


def get_uncertain_based_on(topic):
    topic_count_checkworthy_ans = {}
    topic_count_not_checkworthy_ans = {}
    for k, v in included_answers.items():
        agrees_2 = []
        disagrees_2 = []
        agrees_1 = []
        disagrees_1 = []
        uncertain_disagreed = []
        correct_ans = v[0]['answer']
        claim = v[1][0][5]

        for el in v[1]:  # For each given answer connected to user
            given_ans = el[0]  # Given answer
            if topic == 'gender':
                t = el[1]
            elif topic == 'edu':
                t = el[2]  # This can be natural_science, social_sciences ...
            elif topic == 'age':
                t = el[3]
            elif topic == 'score':
                t = el[4]
            else:
                raise ValueError("Not valid input")

            if given_ans == correct_ans:
                if given_ans == 2:
                    agrees_2.append(t)  # Append topic (f.eks. natural_science) to agrees_2
                elif given_ans == 1:
                    agrees_1.append(t)
            else:
                if given_ans == 2:  # Correct_ans is then 1, and it is therefore appended to the disagrees_1-list
                    disagrees_1.append(t)
                elif given_ans == 1:
                    disagrees_2.append(t)
                else:
                    # print("uncertain-append")
                    uncertain_disagreed.append(t)

        if correct_ans == 2:
            topic_count_checkworthy_ans[k] = [correct_ans, Counter(agrees_2), Counter(disagrees_2),
                                              Counter(uncertain_disagreed), claim]
        elif correct_ans == 1:
            topic_count_not_checkworthy_ans[k] = [correct_ans, Counter(agrees_1), Counter(disagrees_1),
                                                  Counter(uncertain_disagreed), claim]
    ''' # Used to find sentences with counts:
    print("Check-worthy (for {}): ".format(topic))
    for k, v in topic_count_checkworthy_ans.items():
        print("\t{} - {}".format(k, v))
    print("Not check-worthy (for {}): ".format(topic))
    for k, v in topic_count_not_checkworthy_ans.items():
        print("\t{} - {}".format(k, v))
    '''

    one_dict = get_dict_based_on_agreed_ans(topic, topic_count_not_checkworthy_ans)
    two_dict = get_dict_based_on_agreed_ans(topic, topic_count_checkworthy_ans)
    return one_dict, two_dict


def get_dict_based_on_agreed_ans(topic, topic_count_ans):
    res_dict = {}
    for k, v in topic_count_ans.items():
        ans_correct = v[0]
        agreed_count = v[1]
        disagreed_count = v[2]
        uncertain_count = v[3]
        if topic == 'gender':
            for gender in gender_set:
                gender_concat = gender  # + "_" + str(ans_correct)
                if gender_concat not in res_dict:
                    res_dict[gender_concat] = [agreed_count[gender], disagreed_count[gender], uncertain_count[gender]]
                else:
                    res_dict[gender_concat][0] += agreed_count[gender]
                    res_dict[gender_concat][1] += disagreed_count[gender]
                    res_dict[gender_concat][2] += uncertain_count[gender]
        elif topic == 'edu':
            for edu in education_set:
                edu_concat = edu  # + "_" + str(ans_correct)
                if edu_concat not in res_dict:
                    res_dict[edu_concat] = [agreed_count[edu], disagreed_count[edu], uncertain_count[edu]]
                else:
                    res_dict[edu_concat][0] += agreed_count[edu]
                    res_dict[edu_concat][1] += disagreed_count[edu]
                    res_dict[edu_concat][2] += uncertain_count[edu]
        elif topic == 'age':
            for age_group in age_set:
                age_concat = age_group  # + "_" + str(ans_correct)
                if age_concat not in res_dict:
                    res_dict[age_concat] = [agreed_count[age_group], disagreed_count[age_group],
                                           uncertain_count[age_group]]
                else:
                    res_dict[age_concat][0] += agreed_count[age_group]
                    res_dict[age_concat][1] += disagreed_count[age_group]
                    res_dict[age_concat][2] += uncertain_count[age_group]
        elif topic == 'score':
            for score in score_set:
                score_concat = str(score)  # + "_" + str(ans_correct)
                if score_concat not in res_dict:
                    res_dict[score_concat] = [agreed_count[score], disagreed_count[score], uncertain_count[score]]
                else:
                    res_dict[score_concat][0] += agreed_count[score]
                    res_dict[score_concat][1] += disagreed_count[score]
                    res_dict[score_concat][2] += uncertain_count[score]
        else:
            raise ValueError("Not valid input")
    return res_dict


add_result_to_dict()
add_correct_answers_to_dict('_low')
add_correct_answers_to_dict('50')
add_correct_answers_to_dict('75')
add_correct_answers_to_dict('100')


add_agreed_and_uncertain_to_dict()

gender_one_dict, gender_two_dict = get_uncertain_based_on('gender')
db.writeCSVfromDict(gender_one_dict, "results/agreement_correlation_gender1.csv", ['gender', 'agree', 'disagree',
                                                                                   'uncertain'])
db.writeCSVfromDict(gender_two_dict, "results/agreement_correlation_gender2.csv", ['gender', 'agree', 'disagree',
                                                                                   'uncertain'])
# gender_res_dict = {**gender_one_dict, **gender_two_dict}
# print(gender_res_dict)
# db.writeCSVfromDict(gender_res_dict, "results/agreement_correlation_gender.csv", ['gender', 'agree', 'disagree',
# 'uncertain'])

edu_one_dict, edu_two_dict = get_uncertain_based_on('edu')
db.writeCSVfromDict(edu_one_dict, "results/agreement_correlation_education1.csv", ['education', 'agree', 'disagree',
                                                                                   'uncertain'])
db.writeCSVfromDict(edu_two_dict, "results/agreement_correlation_education2.csv", ['education', 'agree', 'disagree',
                                                                                   'uncertain'])
# edu_res_dict = {**edu_one_dict, **edu_two_dict}
# print(edu_res_dict)
# db.writeCSVfromDict(edu_res_dict, "results/agreement_correlation_education.csv", ['education', 'agree', 'disagree',
# 'uncertain'])

age_one_dict, age_two_dict = get_uncertain_based_on('age')
db.writeCSVfromDict(age_one_dict, "results/agreement_correlation_age1.csv", ['age_group', 'agree', 'disagree',
                                                                             'uncertain'])
db.writeCSVfromDict(age_two_dict, "results/agreement_correlation_age2.csv", ['age_group', 'agree', 'disagree',
                                                                             'uncertain'])
# age_res_dict = {**age_one_dict, **age_two_dict}
# print(age_res_dict)
# db.writeCSVfromDict(age_res_dict, "results/agreement_correlation_age.csv", ['age_group', 'agree', 'disagree',
# 'uncertain'])

score_one_dict,score_two_dict  = get_uncertain_based_on('score')
db.writeCSVfromDict(score_one_dict, "results/agreement_correlation_score1.csv", ['score', 'agree', 'disagree',
                                                                                 'uncertain'])
db.writeCSVfromDict(score_two_dict, "results/agreement_correlation_score2.csv", ['score', 'agree', 'disagree',
                                                                                 'uncertain'])
# score_res_dict = {**score_one_dict, **score_two_dict}
# print(score_res_dict)
# db.writeCSVfromDict(score_res_dict, "results/agreement_correlation_score.csv", ['score', 'agree', 'disagree',
# 'uncertain'])
