import time
import numpy as np
from analyze.db import Database

# Agree vertical, disagree horizontal
# Agree: {disagrees...}
matrix_75_0 = {
    'H': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'MH': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'M': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'ML': {'H': False, 'MH': False, 'M': True, 'ML': True, 'L': True},
    'L': {'H': False, 'MH': False, 'M': False, 'ML': False, 'L': False},
}

matrix_75_1 = {
    'H': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'MH': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'M': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'ML': {'H': True, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'L': {'H': False, 'MH': False, 'M': False, 'ML': False, 'L': True},
}

matrix_50_0 = {
    'H': {'H': False, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'MH': {'H': False, 'MH': False, 'M': True, 'ML': True, 'L': True},
    'M': {'H': False, 'MH': False, 'M': False, 'ML': True, 'L': True},
    'ML': {'H': False, 'MH': False, 'M': False, 'ML': False, 'L': False},
    'L': {'H': False, 'MH': False, 'M': False, 'ML': False, 'L': False},
}

matrix_50_1 = {
    'H': {'H': None, 'MH': True, 'M': True, 'ML': True, 'L': True},
    'MH': {'H': False, 'MH': None, 'M': True, 'ML': True, 'L': True},
    'M': {'H': False, 'MH': False, 'M': None, 'ML': True, 'L': True},
    'ML': {'H': False, 'MH': False, 'M': False, 'ML': False, 'L': True},
    'L': {'H': False, 'MH': False, 'M': False, 'ML': False, 'L': False},
}


def getMatrixEvaluation(highest_agreement, agree_class, disagree_class, ans, disagree_ans):
    if highest_agreement >= 0.75:
        if ans == 0 or 0 in disagree_ans:
            print("\t matrix_75_0")
            return matrix_75_0[agree_class][disagree_class]
        else:
            print("\t matrix_75_1")
            return matrix_75_1[agree_class][disagree_class]

    elif 0.5 <= highest_agreement < 0.75:
        if ans == 0 or 0 in disagree_ans:
            print("\t matrix_50_0")
            return matrix_50_0[agree_class][disagree_class]
        else:
            print("\t matrix_50_1")
            return matrix_50_1[agree_class][disagree_class]
    else:
        print("\t else")
        # See .ods
        # if round(highest_agreement, 4) == 0.3333:
        #    return False
        # elif len(set(disagree_ans)) == 1 and ans == 0 or disagree_ans[0] == 0:
        #    return False

        # (0, 0, 1, 2, 2)
        if ans == 0:
            return False
        else:
            return None


def getClassFromAvg(avg):
    if avg > 0.65:
        return 'H'
    elif 0.55 < avg <= 0.65:
        return 'MH'
    elif 0.45 <= avg <= 0.55:
        return 'M'
    elif 0.4 <= avg < 0.45:
        return 'ML'
    else:
        return 'L'


def getValidAnswer(values, answers, highest_agreement):
    # print("Values: ", values)
    # print("Highest agreement: ", highest_agreement)

    ans, agreescore_list, disagree_answer_scorelist = findProbableAnswer(answers, values)

    # print("Agreescorelist and disagreescorelist:  ", agreescore_list, disagree_answer_scorelist)

    agree_class = getClassFromAvg(np.average(agreescore_list))
    disagree_class = getClassFromAvg(np.average([i[1] for i in disagree_answer_scorelist]))

    # print("\t\tagree and disagreeclass: ", agree_class, disagree_class)

    disagree_ans = [a[0] for a in disagree_answer_scorelist]
    # print("Disagree_ans: ", disagree_ans)

    matrix_eval = getMatrixEvaluation(highest_agreement, agree_class, disagree_class, ans, disagree_ans)
    # print("\t\tMatrixEval: ", matrix_eval)

    if matrix_eval is None:
        avg_agree = np.average(agreescore_list)
        avg_disagree = np.average([s[1] for s in disagree_answer_scorelist])
        if (avg_agree - avg_disagree) / avg_disagree > 0.2:
            return ans
        else:
            return -1
    elif matrix_eval:
        return ans
    else:
        return -1


def findProbableAnswer(answers, new_values):
    avg = -1
    ans = -1
    ans_info_dict = {}
    for a in answers:
        tmp_agree_score_list = [i[1] for i in list(filter((lambda x: x[0] == a), new_values))]
        tmp_disagree_score_list = [(i[0], i[1]) for i in list(filter((lambda x: x[0] != a), new_values))]
        temp_avg = np.average(tmp_agree_score_list)
        ans_info_dict[a] = [temp_avg, tmp_agree_score_list, tmp_disagree_score_list,
                            np.average(tmp_agree_score_list)]
        if temp_avg > avg:
            avg = temp_avg
            ans = a
    agree_score_list = ans_info_dict[ans][1]  # "Riktige" svaret - vil ha svart det samme, nemlig ans
    disagree_score_list = ans_info_dict[ans][2]  # De som er uenige,

    return ans, agree_score_list, disagree_score_list


def getLabelWithHighestAgreement(values):
    answer_list = [i[0] for i in values]
    # print('Answer list: ', answer_list)
    answer_percent = [(answer_list.count(i) / len(answer_list)) for i in range(4)]
    highest_agreement = max(answer_percent)
    answers = np.where(np.array(answer_percent) == highest_agreement)[0].tolist()

    # print("\t\thighest_agreement1: ", highest_agreement)
    # print("\t\tAnswers: ", answers)

    if highest_agreement == 1.0:
        if answers[0] == 3:
            return highest_agreement, -1
        else:
            return highest_agreement, answers[0]
    elif highest_agreement >= 0.75 and answers[0] == 3:
        return highest_agreement, -1
    else:
        new_values = list(filter(lambda x: x[0] != 3, values))
        answer_list = [i[0] for i in new_values]
        # print('Answer list: ', answer_list)
        answer_percent = [(answer_list.count(i) / len(answer_list)) for i in range(4)]
        # print("Percent: ", answer_percent)
        highest_agreement = max(answer_percent)
        # print("\t\thighest_agreement2: ", highest_agreement)
        answers = np.where(np.array(answer_percent) == highest_agreement)[0].tolist()
        # print('Answers: ', answers)

        if highest_agreement == 1.0:
            return highest_agreement, answers[0]
        else:
            return round(highest_agreement, 2), getValidAnswer(new_values, answers, highest_agreement)


db = Database('final_ccdb01.db')

question_dict = {}

answer_userscore_dict = {}

answer_userscore_matrix = db.fetch('SELECT a.id, a.unlabeled_id, a.answer, us.score '
                                   'FROM Answers as a INNER JOIN Users as us on a.username=us.username '
                                   'WHERE a.unlabeled_id IS NOT NULL AND a.unlabeled_id in '
                                   '( SELECT an.unlabeled_id '
                                   'FROM Answers as an '
                                   'WHERE an.unlabeled_id IS NOT NULL '
                                   'GROUP BY an.unlabeled_id HAVING count(an.answer) > 3) '
                                   'ORDER BY a.unlabeled_id ASC, us.score DESC, a.answer ASC')

# print(answer_userscore_matrix)

questions = db.fetch('SELECT * FROM UnlabeledClaims')
for row in questions:
    question_dict[row[0]] = row[1]

for row in answer_userscore_matrix:
    aid = row[0]
    question_id = row[1]
    answer = row[2]
    score = row[3]
    if question_id not in answer_userscore_dict.keys():
        answer_userscore_dict[question_id] = [(answer, score)]
    else:
        answer_userscore_dict[question_id].append((answer, score))


# print(answer_userscore_dict)

agreement100 = {}
agreement75 = {}
agreement50 = {}
agreement_low = {}

for key, value in answer_userscore_dict.items():
    print(key)
    rate, agreement = getLabelWithHighestAgreement(value)
    print()
    if rate == 1.0:
        # 100% agreement
        agreement100[key] = [agreement, rate, question_dict[key]]
    elif 0.75 <= rate < 1.0:
        # 75% - 100% agreement
        agreement75[key] = [agreement, rate, question_dict[key]]
    elif 0.5 <= rate < 0.75:
        # 50% - 75% agreement
        agreement50[key] = [agreement, rate, question_dict[key]]
    else:
        # lower than 50% agreement
        agreement_low[key] = [agreement, rate, question_dict[key]]


db.writeCSVfromDict(agreement100, 'results/matrix_agreement_score100.csv', ['unlabeled_id', 'label', 'rate', 'claim'])
db.writeCSVfromDict(agreement75, 'results/matrix_agreement_score75.csv', ['unlabeled_id', 'label', 'rate', 'claim'])
db.writeCSVfromDict(agreement50, 'results/matrix_agreement_score50.csv', ['unlabeled_id', 'label', 'rate', 'claim'])
db.writeCSVfromDict(agreement_low, 'results/matrix_agreement_score_low.csv', ['unlabeled_id', 'label', 'rate', 'claim'])
