import sqlite3, os, csv


class Database:

    def __init__(self, name=None):

        self.conn = None
        self.cursor = None

        if name:
            self.open(name)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def open(self, name):
        try:
            self.conn = sqlite3.connect(name)
            self.cursor = self.conn.cursor()

        except sqlite3.Error as e:
            print("Error: ", e)

    def close(self):

        if self.conn:
            self.conn.commit()
            self.cursor.close()
            self.conn.close()

    def query(self, sql):
        self.cursor.execute(sql)

    def fetch(self, sql):
        self.cursor.execute(sql)
        return self.cursor.fetchall()

    def fetchWithParam(self, sql, param):
        self.cursor.execute(sql, param)
        return self.cursor.fetchall()

    @staticmethod
    def writeCSV(data, file_name: str, header: list=None):
        folder = os.listdir(os.curdir)
        if file_name in folder:
            raise Exception('Duplicate filename')
        else:
            with open(file_name, 'a', encoding='utf-8') as out_file:
                writer = csv.writer(out_file, delimiter=',', lineterminator='\n')
                if header is not None:
                    writer.writerow(header)
                for item in data:
                    writer.writerow(item)

    @staticmethod
    def writeCSVfromDict(data, file_name: str, header: list=None):
        folder = os.listdir(os.curdir)
        if file_name in folder:
            raise Exception('Duplicate filename')
        else:
            with open(file_name, 'a', encoding='utf-8') as out_file:
                writer = csv.writer(out_file, delimiter=',', lineterminator='\n')
                if header is not None:
                    writer.writerow(header)
                for key, value in data.items():
                    if isinstance(key, tuple):
                        if isinstance(value, dict):
                            writer.writerow([k for k in key] + [v for v in value.values()])
                    else:
                        if isinstance(value, list):
                            if isinstance(value[0], list):
                                for v in value:
                                    writer.writerow([key] + v)
                            else:
                                writer.writerow([key] + value)
                        elif isinstance(value, dict):
                            writer.writerow([key] + [v for v in value.values()])
                        elif isinstance(value, int) or isinstance(value, str):
                            writer.writerow([key, value])
                        else:
                            raise TypeError('value is not of type list')

