from analyze.db import Database
import csv
import time

db = Database('final_ccdb01.db')

question_dict = {}

questions = db.fetch('SELECT * FROM UnlabeledClaims')
for row in questions:
    question_dict[row[0]] = row[2]


agreed_answer_dict = {}
uncertain_answer_dict = {}


def add_labels_to_dict(score_num):
    with open('results/matrix_agreement_score{}.csv'.format(score_num), 'r', encoding='utf-8') as in_file:
        reader = csv.reader(in_file)
        header = next(reader)
        # print(header)
        for r in reader:
            qid, agreed_a, rate, q = r
            qid = int(float(qid))
            if int(float(agreed_a)) == -1:
                if qid not in uncertain_answer_dict:
                    uncertain_answer_dict[qid] = {'label': int(float(agreed_a)), 'external_id': question_dict[qid]}
                else:
                    print("In dict!")
            else:
                if qid not in agreed_answer_dict:
                    agreed_answer_dict[qid] = {'label': int(float(agreed_a)), 'external_id': question_dict[qid]}
                else:
                    print('Is in dict!')
                # print('{}\t{}\t{}\t{}'.format(qid, agreed_answer, rate, question))


add_labels_to_dict('_low')
add_labels_to_dict('50')
add_labels_to_dict('75')
add_labels_to_dict('100')

party_dict = {}


def make_party_dict():
    with open('../ton/ton.csv', 'r', encoding='utf-8') as in_file:
        reader = csv.reader(in_file)
        header = next(reader)
        # print(header)
        for r in reader:
            t_id = r[0]
            p_id = r[11] if len(r[11]) != 0 else 'NA'
            party = r[12]
            party_dict[t_id] = (party, p_id)
            # print("{}: {} ({})".format(t_id, party, p_id))


print("Starting to make party_dict")
start = time.clock()
make_party_dict()
print("Finished making party_dict in {} sek".format(time.clock()-start))
agreed_answer_dict_with_p = {}
uncertain_answer_dict_with_p = {}


def add_party_to_dicts():
    for key, value in agreed_answer_dict.items():
        external_id = value['external_id'][:-5]
        agreed_ans = value['label']
        party = party_dict[external_id][0]
        p_id = party_dict[external_id][1]
        agreed_answer_dict_with_p[key] = [agreed_ans, p_id, party, value['external_id']]

    for k, v in uncertain_answer_dict.items():
        external_id = v['external_id'][:-5]
        agreed_ans = v['label']
        party = party_dict[external_id][0]
        p_id = party_dict[external_id][1]
        uncertain_answer_dict_with_p[k] = [agreed_ans, p_id, party, v['external_id']]

    # print(agreed_answer_dict_with_p)
    # print(uncertain_answer_dict_with_p)


add_party_to_dicts()
'''
db.writeCSVfromDict(agreed_answer_dict_with_p, "results/party_classify_agreed.csv",
                    ['unlabeled_id', 'agreed_answer', 'party_id', 'party_name', 'external_id'])
db.writeCSVfromDict(uncertain_answer_dict_with_p, "results/party_classify_uncertain.csv",
                   ['unlabeled_id', 'agreed_answer', 'party_id', 'party_name', 'external_id'])
'''
party_answer_type_dict = {}


def make_party_answer_type_dict():
    for key, value in agreed_answer_dict_with_p.items():
        agreed_ans, p_id, party, external_id = value
        if p_id not in party_answer_type_dict:
            party_answer_type_dict[p_id] = {'NV': 0, 'NCWV': 0, 'CWV': 0, 'uncertain': 0, 'total': 0}
        if agreed_ans == 0:
            party_answer_type_dict[p_id] = {'NV': (party_answer_type_dict[p_id]['NV']+1),
                                            'NCWV': party_answer_type_dict[p_id]['NCWV'],
                                            'CWV': party_answer_type_dict[p_id]['CWV'],
                                            'uncertain': party_answer_type_dict[p_id]['uncertain'],
                                            'total': party_answer_type_dict[p_id]['total']+1}
        elif agreed_ans == 1:
            party_answer_type_dict[p_id] = {'NV': party_answer_type_dict[p_id]['NV'],
                                            'NCWV': (party_answer_type_dict[p_id]['NCWV']+1),
                                            'CWV': party_answer_type_dict[p_id]['CWV'],
                                            'uncertain': party_answer_type_dict[p_id]['uncertain'],
                                            'total': party_answer_type_dict[p_id]['total']+1}
        elif agreed_ans == 2:
            party_answer_type_dict[p_id] = {'NV': party_answer_type_dict[p_id]['NV'],
                                            'NCWV': party_answer_type_dict[p_id]['NCWV'],
                                            'CWV': (party_answer_type_dict[p_id]['CWV']+1),
                                            'uncertain': party_answer_type_dict[p_id]['uncertain'],
                                            'total': party_answer_type_dict[p_id]['total']+1}
        else:
            print("{}: agreed_ans: {}".format(p_id, agreed_ans))
            if p_id not in party_answer_type_dict:
                party_answer_type_dict[p_id] = {'NV': 0, 'NCWV': 0, 'CWV': 0, 'uncertain': 0, 'total': 0}
            party_answer_type_dict[p_id] = {'NV': party_answer_type_dict[p_id]['NV'],
                                            'NCWV': party_answer_type_dict[p_id]['NCWV'],
                                            'CWV': (party_answer_type_dict[p_id]['CWV']),
                                            'uncertain': party_answer_type_dict[p_id]['uncertain'] + 1,
                                            'total': party_answer_type_dict[p_id]['total'] + 1}

    for k, v in uncertain_answer_dict_with_p.items():
        agreed_ans, p_id, party, external_id = v
        if p_id not in party_answer_type_dict:
            party_answer_type_dict[p_id] = {'NV': 0, 'NCWV': 0, 'CWV': 0, 'uncertain': 0, 'total': 0}
        party_answer_type_dict[p_id] = {'NV': party_answer_type_dict[p_id]['NV'],
                                        'NCWV': party_answer_type_dict[p_id]['NCWV'],
                                        'CWV': (party_answer_type_dict[p_id]['CWV']),
                                        'uncertain': party_answer_type_dict[p_id]['uncertain']+1,
                                        'total': party_answer_type_dict[p_id]['total']+1}


make_party_answer_type_dict()
print(party_answer_type_dict)
db.writeCSVfromDict(party_answer_type_dict, "results/party_classify_answer.csv",
                    ['party_id', 'NV', 'NCWV', 'CWV', 'uncertain', 'total'])
party_agreed_or_not_dict = {}


def make_party_agreed_or_not_dict():
    for key, value in agreed_answer_dict_with_p.items():
        agreed_ans, p_id, party, external_id = value
        if p_id not in party_agreed_or_not_dict:
            party_agreed_or_not_dict[p_id] = {'agreed': 1, 'uncertain': 0, 'total': 1}
        else:
            party_agreed_or_not_dict[p_id] = {'agreed': (party_agreed_or_not_dict[p_id]['agreed']+1),
                                              'uncertain': party_agreed_or_not_dict[p_id]['uncertain'],
                                              'total': (party_agreed_or_not_dict[p_id]['total']+1)}
    for k, v in uncertain_answer_dict_with_p.items():
        agreed_ans, p_id, party, external_id = v
        if p_id not in party_agreed_or_not_dict:
            party_agreed_or_not_dict[p_id] = {'agreed': 0, 'uncertain': 1, 'total': 1}
        else:
            party_agreed_or_not_dict[p_id] = {'agreed': party_agreed_or_not_dict[p_id]['agreed'],
                                              'uncertain': (party_agreed_or_not_dict[p_id]['uncertain']+1),
                                              'total': (party_agreed_or_not_dict[p_id]['total'] + 1)}


make_party_agreed_or_not_dict()
print(party_agreed_or_not_dict)
db.writeCSVfromDict(party_agreed_or_not_dict, "results/party_classify_agreed.csv", ['party_id', 'agreed', 'uncertain', 'total'])

# Solves: "Which parties had most 0, 1 and 2 sentences included", and "Which party was "easiest/hardest" to classify??"
