import csv, sqlite3, nltk, time, os, datetime, random
import pandas as pd

text_items = []


def extract_existing_answers_from_db():
    res_dict = {}

    con = sqlite3.connect('ccdb01.db')
    try:
        query = "SELECT q.claim, q.external_id from UnlabeledClaims as q inner join Answers as a on q.id=a.unlabeled_id"
        cursor = con.cursor()
        cursor.execute(query)

        rows = cursor.fetchall()

        for row in rows:
            res_dict[row[1][:-5]] = [row[1], row[0]]

        print("Length of res_dict: {}".format(len(res_dict)))
        con.commit()
        return res_dict
    except sqlite3.Error as e:
        print('ERROR: ', e)
    finally:
        con.close()


already_in_db = extract_existing_answers_from_db()


def previous_current_next(iterable):
    """Make an iterator that yields an (previous, current, next) tuple per element.

    Returns None if the value does not make sense (i.e. previous before
    first and next after last).
    """
    iterable = iter(iterable)
    prv = None
    cur = next(iterable)
    try:
        while True:
            nxt = next(iterable)
            yield (prv, cur, nxt)
            prv = cur

            cur = nxt
    except StopIteration:
        yield (prv, cur, None)


def fix_bad_splits(id, sentences):
    pcn = previous_current_next(sentences)
    result = []
    for i in pcn:
        prev, cur, nxt = i
        try:
            if cur[0].islower():
                result.remove(prev)
                result.append(prev + " " + cur)
            else:
                result.append(cur)
        except ValueError as e:
            print('Error: {}, FOR: \n\t{}'.format(e, i))

    return result


# Use nltk to tokenize text into sentences
def split_text_to_sentences(id, text):
    tokenizer = nltk.data.load('tokenizers/punkt/norwegian.pickle')
    tokens = tokenizer.tokenize(text)
    # print('TOKENS:     ', tokens)
    # print('Length: ', len(tokens))
    filtered_tokens = list(filter(lambda x: 6 <= len(x.split()) <= 45, tokens))  # Remove sentences shorter than 5 words
    # print('TOKENS2: ', filtered_tokens)
    # print('Length: ', len(filtered_tokens))
    # print()
    no_question_tokens = list(filter(lambda x: not x.endswith('?'), filtered_tokens))  # Remove questions
    no_exclamation_tokens = list(filter(lambda x: not x.endswith('!'), no_question_tokens))
    fixed_tokens = fix_bad_splits(id, no_exclamation_tokens)
    return fixed_tokens


def shuffle_rows():
    path = 'party_split/'
    folder = os.listdir(path)
    if not folder:
        raise FileNotFoundError('Folder "{}" is empty, have you run "write_party_based()"?'.format(path))
    for file in folder:
        df = pd.read_csv(path+file, header=None, encoding='utf-8', sep=',')
        ds = df.sample(frac=1)
        ds.to_csv('party_shuffled/shuffled_{}'.format(file[19:]), encoding='utf-8', sep=',', header=None, index=False)


yearmonth_set = set()
ym_dict = {}

removable_any_words = ['voter', 'vil ta sete', 'tatt sete', '...',
                       'jeg mener', 'jeg mente', 'jeg synes', 'jeg ønsker',
                       'eg meiner', 'eg meinte', 'eg synest', 'eg ynskjer',
                       'vi mener', 'vi mente', 'vi synes', 'vi ønsker', 'vi ynskjer',
                       'skjønn ', ' skjønn', 'spørsmål',
                       'på vegne', 'representanten']#, 'på vegne av', 'jeg takker', 'jeg viser', 'jeg har lyst', 'jeg er glad']  # All sentences that contains ANY of these phrases will be removed

# A lot of "Spørsmål x (int)" is present. Add them to removable_any_words
for i in range(1, 16):
    removable_any_words.append('spørsmål ' + str(i))


def add_to_yearmonth_set():
    path = 'party_split/'
    folder = os.listdir(path)
    # Velg så Y innen hvert parti fra hver session
    # Sjekk samtidig at man får Z forskjellige representanter
    for file in folder:
        with open(path + file, 'r', encoding='utf-8') as party_file:
            reader = csv.reader(party_file)
            for row in reader:
                date = row[4]
                yearmonth = date[:7]
                yearmonth_set.add(yearmonth)
                # 5 setninger for hver year-month


def get_representative_selection():
    path = 'party_shuffled/'
    folder = os.listdir(path)
    if not folder:
        raise FileNotFoundError('Folder "{}" is empty, have you run "shuffle_rows()"?'.format(path))
    # Velg så Y innen hvert parti fra hver session
    # Sjekk samtidig at man får Z forskjellige representanter

    for file in folder:
        result = []
        print('format: ', file[9:])
        file_to_write = 'party_chosen/chosen_{}'.format(file[9:])
        print('File to write: ', file_to_write)
        for item in yearmonth_set: # Adds each yearmonth to a dict
            ym_dict[item] = 0
        for key, value in already_in_db.items():
            ym_dict[value[2][:7]] += 1
        with open(path+file, 'r', encoding='utf-8') as party_file:
            reader = csv.reader(party_file)
            for row in reader:
                date = row[4]
                party = row[2]
                yearmonth = date[:7]
                current_count = ym_dict[yearmonth]
                if current_count < 7:
                    result.append(row)
                    ym_dict[yearmonth] += 1

                # 5 setninger for hver year-month
        with open(file_to_write, 'a', encoding='utf-8') as party_out_file:
            writer = csv.writer(party_out_file, delimiter=',', lineterminator='\n')
            for r in result:
                writer.writerow(r)


party_list = ['A', 'FrP', 'H', 'Kp', 'KrF', 'NA', 'Sp', 'SV', 'TF', 'Uavhengig', 'V']


def write_party_based():
    folder = os.listdir('party_filtered')
    if not folder:
        raise FileNotFoundError('Folder "{}" is empty, have you run "filter_ton.py"?'.format('party_filtered'))
    for file in folder:
        print('Writing file: {}'.format(file))
        with open('party_filtered/{}'.format(file), 'r', encoding='utf-8') as ton_file, open('party_split/split_{}'.format(file), 'a', encoding='utf-8') as ton_out_file,\
            open('party_short/short_split_{}'.format(file), 'a', encoding='utf-8') as ton_short_file:
            reader = csv.reader(ton_file)
            writer = csv.writer(ton_out_file, delimiter=',', lineterminator='\n')
            short_writer = csv.writer(ton_short_file, delimiter=',', lineterminator='\n')
            next(reader)
            for row in reader:
                id = row[0]
                party = row[1]
                rep_name = row[2]
                date = row[3]
                text = row[4]
                sentences = split_text_to_sentences(id, text)
                id_counter = 0
                for sent in sentences:
                    if any(w in sent.lower() for w in removable_any_words):
                        continue  # if the sentence as any word from removable_any_words, jump to next iteration
                    # final_list.append([id, sentences])
                    if len(sent.split()) < 9:
                        uid = id + '-{0:.3f}'.format(id_counter / 1000).replace('.', '')  # Generate id for each item
                        short_writer.writerow([id, uid, party, rep_name, date, sent])
                    if sent[0].isalnum() and sent[0].isupper():
                        uid = id + '-{0:.3f}'.format(id_counter / 1000).replace('.', '')  # Generate id for each item
                        writer.writerow([id, uid, party, rep_name, date, sent])
                        id_counter += 1


def deleteAll(table):
    con = sqlite3.connect('ccdb01.db')
    try:
        query = 'DELETE FROM {}'.format(table)
        cursor = con.cursor()
        cursor.execute(query)
        con.commit()
    except sqlite3.Error as e:
        print('ERROR: ',e)
    finally:
        con.close()


def add_to_database(object):
    print('Adding to database...')
    con = sqlite3.connect('ccdb01.db')
    try:
        print('is instance')
        cursor = con.cursor()
        # for item in object:
        #    text_items.append(item)
        #    print('Inserting item: ', item)
        #    print('Inserting item (tuple): ', item,)
        # cursor.execute('INSERT INTO Questions (question,external_id) VALUES (?,?)', item)
        cursor.executemany('INSERT INTO Questions (question,external_id) VALUES (?,?)', object)
        con.commit()

    except sqlite3.Error as e:
        print('ERROR: ',e)
    finally:
        print('Done.')
        print()
        con.close()


def write_split_to_db():
    folder = os.listdir('party_chosen')
    sentence_list = []

    for file in folder:
        print('Writing file to db: ', file)
        with open('party_chosen/{}'.format(file), 'r', encoding='utf-8') as db_file:
            reader = csv.reader(db_file)
            # db_obj = []
            for row in reader:
                if row[5] in already_in_db:
                    print(row[5])
                else:
                    sentence_list.append((row[5], row[1]))
                # print(row)
                # db_obj.append((row[5], row[1]))
            # add_to_database(db_obj)
            print('Done writing: {}'.format(file))
    # randomize sentence_list
    random.shuffle(sentence_list)
    add_to_database(sentence_list)


def get_date_from_external_id():
    with open('ton.csv', 'r', encoding='utf-8') as ton_file:
        reader = csv.reader(ton_file)
        next(reader)  # skip header
        for row in reader:
            id = row[0]
            date = row[81]
            if id in already_in_db:
                already_in_db[id].append(date)


# write_split_to_db()
# deleteAll('Questions')
# write_short()
# write_all()
'''
start = time.clock()
write_party_based()
end = time.clock()
print('Write party: {}'.format(end-start))

start = time.clock()
shuffle_rows()
end = time.clock()
print('Shuffle rows: {}'.format(end-start))
'''
start = time.clock()
add_to_yearmonth_set()
end = time.clock()
print('Add to yearmonth: {}'.format(end-start))

get_date_from_external_id()
print(already_in_db)

start = time.clock()
get_representative_selection()
end = time.clock()
print('get representative selection: {}'.format(end-start))

write_split_to_db()


