import React, { Component } from 'react';
import { Navbar, Button, MenuItem, Dropdown } from 'react-bootstrap';
import './App.css';
import axios from 'axios';
import URLS from './urls';
import {logging} from "./Constants/Logging";
import Footer from "./Footer";
//prev login: TriveligLøvgrønnRødhalsgås
class App extends Component {

    constructor(){
        super();
        this.state = {
            user: null,
            winwidth: 0,
            winheight: 0
        };

        this.getUser = null;
        this.confirmationMessage = "Denne siden ber deg bekrefte at du ønsker å forlate siden - data som du allerede har skrevet inn vil ikke lagres."
    }

    componentWillMount(){
        this.updateDimensions();
    }

    updateDimensions() {
        if((this.state.winheight<550 && window.innerHeight > 550) || (this.state.winheight>550 && window.innerHeight<550)){
            //console.log("\tRerender app.js because of height!, setting heigt to: ", window.innerHeight);
            this.setState({winheight: window.innerHeight});
        }
        if((this.state.winwidth<768 && window.innerWidth>768) || (this.state.winwidth>768 && window.innerWidth<768)){
            //console.log("\tRerender app.js because of width!, setting width to: ", window.innerWidth);
            this.setState({winwidth: window.innerWidth});
        }
    }

    componentDidMount(){
        window.addEventListener("resize", this.updateDimensions.bind(this));
        const { isAuthenticated } = this.props.auth;
        if(isAuthenticated()){
            this.fetchUser();
        }
    }

    componentWillUnmount(){
        window.removeEventListener("resize", this.updateDimensions);
    }

    fetchUser(){
        const {getUser} = this.props.auth;
        const {getAccessToken} = this.props.auth;

        if(getUser()){
            this.setState({user: getUser()});
        }
        else {
            let self = this;
            const {getUserInit} = this.props.auth;
            getUserInit((err, user) => {
                if(user){
                    self.setState({user: user});
                }
                else {
                    self.setState({user: null});
                    //self.props.auth.logout();
                    //console.error("No user in app: ", err);
                    logging(getAccessToken()).error("Error occured in fetchUser in App.js", err);
                    //defaultLogging().error("Error occured in fetchUser in App.js", error);

                }
            })
        }
    }

    goTo(route, fromAuth) {
        /*console.log("Current route: " + this.props.history.location.pathname);
        console.log("Next route: " + "/"+route);*/

        const { isAuthenticated } = this.props.auth;
        const {setAuthError} = this.props.auth;

        if (!isAuthenticated() && fromAuth) {
            this.setState({user: null}, function () {
                this.props.auth.logout("valid_logout");
            })
        }

        if(this.props.history.location.pathname === ("/classification") && isAuthenticated()){
            if(window.confirm(this.confirmationMessage)){
                this.props.history.replace(`/${route}`);
                setAuthError(null);
            }
        }
        else if(this.props.history.location.pathname !== ("/"+route)){
            this.props.history.replace(`/${route}`);
            setAuthError(null);
        }
    }

    /*login() {
        this.props.auth.login();
    }*/

    logout() {
        if(this.props.history.location.pathname === ("/classification")){
            if(window.confirm(this.confirmationMessage)){
                this.setState({user: null}, function () {
                    this.props.auth.logout("valid_logout");
                })
            }
        }else {
            this.setState({user: null}, function () {
                this.props.auth.logout("valid_logout");
            })
        }
    }

    /**
     * Is equal to goTo(), except it sends scores to the page
     * @param route
     * @param scores
     */
    goToTopscore(route, scores) {
        //this.props.history.replace(`/${route}`)
        const { getUser } = this.props.auth;
        let user = null;

        if(this.state.user) {
            user = this.state.user;
        }else if(getUser()){
            user = getUser();
        }

        const { isAuthenticated } = this.props.auth;

        if(this.props.history.location.pathname === ("/classification") && isAuthenticated()){
            if(window.confirm(this.confirmationMessage)){
                this.props.history.replace({
                    pathname:`/${route}`,
                    state: {
                        scores: scores,
                        user: user
                    }
                });
            }
        }
        else if(this.props.history.location.pathname !== ("/"+route)){
            this.props.history.replace({
                pathname:`/${route}`,
                state: {
                    scores: scores,
                    user: user
                }
            });
        }
    }

    gotoCreateUser(route, uname){
        this.props.history.replace({
            pathname:`/${route}`,
            state: {
                username: uname
            }
        });
    }

    generateUsername (){
        const {setAuthError} = this.props.auth;
        if(this.props.history.location.pathname !== ("/create-user")){
            setAuthError(null);
            let self = this;
            let headers = {
                'Content-Type': 'application/json',
                'Access-Control-Allow-Origin': URLS.ssl_app_url,
                'Access-Control-Allow-Credentials': true,
                'Button-Event': 'dkjfgh908u3jrnfp89dyduh34gbhjait78',
            };
            // axios request
            axios({
                url: `${URLS.callable_api_url}/generate-username`,
                method: 'get',
                headers: headers
            }).then((response) => {
                let username = response.data.username;
                self.gotoCreateUser('create-user', username)
            }).catch((error) => {
                //console.error('error: ', error);
                //logging(getAccessToken()).error("Error occured in getScoreList in App.js", error);
                //defaultLogging().error("Error occured in getScoreList in App.js", error);
            })
        }
    }

    getScoreList() {
        const {getAccessToken} = this.props.auth;
        const {isAuthenticated} = this.props.auth;

        if (!isAuthenticated()){
            this.setState({user: null}, function () {
                this.props.auth.logout("invalid_token");
            })
        }
        else {
            let self = this;
            let headers = {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${getAccessToken()}`,// + access_token,
                'Access-Control-Allow-Origin': URLS.ssl_app_url,
                'Access-Control-Allow-Credentials': true
            };
            // axios request
            axios({
                url: `${URLS.callable_api_url}/scores`,
                method: 'get',
                headers: headers
            }).then((response) => {
                self.goToTopscore('results', response.data)
            }).catch((error) => {
                logging(getAccessToken()).error("Error occured in getScoreList in App.js", error);
                //defaultLogging().error("Error occured in getScoreList in App.js", error);
            })
        }
    }

    setActive(route) {
        let current_route = this.props.history.location.pathname;
        return current_route === `/${route}`;
    }


    render() {
        const { isAuthenticated } = this.props.auth;
        const { getUser } = this.props.auth;

        return (
            <div>
                {window.innerHeight > 550 && window.innerWidth > 768 ?
                    <div>
                        <Navbar fluid>
                            <Navbar.Header>
                                <Navbar.Brand>
                                    <a href={URLS.ssl_app_url}>ClaimCollector</a>
                                </Navbar.Brand>
                            </Navbar.Header>

                            <div className="header-div">
                                <div className="nav-lefty">

                                    {
                                        isAuthenticated() && (
                                            <div>
                                                <Button
                                                    id="btn-home"
                                                    bsStyle="primary"
                                                    className="btn-margin"
                                                    onClick={this.goTo.bind(this, 'home', true)}
                                                    active={this.setActive('home')}
                                                >
                                                    <i className="fa fa-home" aria-hidden="true"></i> Startside
                                                </Button>
                                                <Button
                                                    id="btn-topscore"
                                                    bsStyle="primary"
                                                    className="btn-margin"
                                                    onClick={this.getScoreList.bind(this)}
                                                    active={this.setActive('topscore')}
                                                >
                                                    <i className="fa fa-list-ol" aria-hidden="true"></i> Resultater
                                                </Button>
                                            </div>
                                        )
                                    }

                                    {
                                        !isAuthenticated() && (
                                            <Button
                                                id="btn-home"
                                                bsStyle="primary"
                                                className="btn-margin"
                                                onClick={this.goTo.bind(this, 'home', false)}
                                                active={this.setActive('home')}
                                            >
                                                <i className="fa fa-home" aria-hidden="true"></i> Startside
                                            </Button>
                                        )
                                    }
                                </div>

                                <div className="nav-righty">
                                    {
                                        !isAuthenticated() && (
                                            <div>
                                                <Button
                                                    bsStyle="primary"
                                                    className="btn-margin"
                                                    onClick={this.generateUsername.bind(this)}
                                                >
                                                    <i className="fa fa-plus-circle" aria-hidden="true"></i> Lag bruker
                                                </Button>

                                                <Button
                                                    bsStyle="primary"
                                                    className="btn-margin"
                                                    onClick={this.goTo.bind(this, 'login', false)}
                                                >
                                                    <i className="fa fa-sign-in" aria-hidden="true"></i> Logg Inn
                                                </Button>
                                            </div>
                                        )
                                    }
                                    {
                                        isAuthenticated() && (
                                            <div>
                                                <Navbar.Brand>
                                                    {this.state.user ?  //If state has user
                                                        <div><i className="fa fa-user" aria-hidden="true"></i> {this.state.user.username}</div> : //show this user
                                                        <div> {getUser() ? //Else see if we can get the user
                                                            <div><i className="fa fa-user" aria-hidden="true"></i> {getUser().username}</div> : //show this user
                                                            null //Else show nothing here
                                                        }
                                                        </div>
                                                    }
                                                </Navbar.Brand>
                                                <Button
                                                    bsStyle="primary"
                                                    className="btn-margin"
                                                    onClick={this.logout.bind(this)}
                                                >
                                                    <i className="fa fa-sign-out" aria-hidden="true"></i> Logg Ut
                                                </Button>
                                            </div>
                                        )
                                    }
                                </div>
                            </div>
                        </Navbar>
                        <Footer/>
                    </div>
                    :
                    <Navbar fluid>
                        <Navbar.Header>
                            {this.state.user ?  //If state has user
                                <Navbar.Brand className="twolines_navbar"><a href={URLS.ssl_app_url}>ClaimCollector</a><br/> <i className="fa fa-user" aria-hidden="true"></i>{this.state.user.username}</Navbar.Brand> : //show this user
                                <div> {getUser() ? //Else see if we can get the user
                                    <Navbar.Brand className="twolines_navbar"><a href={URLS.ssl_app_url}>ClaimCollector</a><br/> <i className="fa fa-user" aria-hidden="true"></i>{getUser().username}</Navbar.Brand> : //show this user
                                    <Navbar.Brand><a href={URLS.ssl_app_url}>ClaimCollector</a></Navbar.Brand> //Else show logo only
                                }
                                </div>
                            }
                        </Navbar.Header>

                        <div className="header-div">
                            <div className="nav-lefty">
                            </div>
                            <div className="nav-righty">
                                {
                                    !isAuthenticated() && (
                                        <Dropdown id="dropdown1" className="btn-margin" pullRight>
                                            <Dropdown.Toggle>
                                                <i className="fa fa-bars" aria-hidden="true"></i>
                                            </Dropdown.Toggle>
                                            <Dropdown.Menu className="">
                                                <MenuItem eventKey="1" onSelect={this.goTo.bind(this, 'home', false)}>Startside</MenuItem>

                                                <MenuItem divider />
                                                <MenuItem eventKey="2" onSelect={this.generateUsername.bind(this)}>Lag bruker</MenuItem>
                                                <MenuItem eventKey="3" onSelect={this.goTo.bind(this, 'login', false)}>Logg inn</MenuItem>

                                                < MenuItem divider />
                                                <MenuItem title={URLS.ssl_app_url+'/privacy'} href={URLS.ssl_app_url+'/privacy'} target="_blank">Personvernerklæring</MenuItem>
                                                <MenuItem title={URLS.ssl_app_url+'/terms-and-conditions'} href={ URLS.ssl_app_url+'/terms-and-conditions'} target="_blank">Vilkår for bruk</MenuItem>
                                                <MenuItem href="mailto:claimcollector@gmail.com?Subject=Generelle%20spørsmål%20om%20ClaimCollector">Kontakt: claimcollector@gmail.com</MenuItem>
                                            </Dropdown.Menu>
                                        </Dropdown>

                                    )
                                }

                                {
                                    isAuthenticated() && (
                                        <Dropdown id="dropdown2" className="btn-margin" pullRight>
                                            <Dropdown.Toggle>
                                                <i className="fa fa-bars" aria-hidden="true"></i>
                                            </Dropdown.Toggle>
                                            <Dropdown.Menu className="">
                                                <MenuItem eventKey="1" onSelect={this.goTo.bind(this, 'home', true)}>Startside</MenuItem>
                                                <MenuItem eventKey="4" onSelect={this.getScoreList.bind(this)}>
                                                    Resultater</MenuItem>

                                                < MenuItem divider />
                                                <MenuItem eventKey="5" onSelect={this.logout.bind(this)}>
                                                    Logg ut</MenuItem>

                                                < MenuItem divider />
                                                <MenuItem title={URLS.ssl_app_url+'/privacy'} href={URLS.ssl_app_url+'/privacy'} target="_blank">Personvernerklæring</MenuItem>
                                                <MenuItem title={URLS.ssl_app_url+'/terms-and-conditions'} href={ URLS.ssl_app_url+'/terms-and-conditions'} target="_blank">Vilkår for bruk</MenuItem>
                                                <MenuItem href="mailto:claimcollector@gmail.com?Subject=Generelle%20spørsmål%20om%20ClaimCollector">Kontakt: claimcollector@gmail.com</MenuItem>
                                            </Dropdown.Menu>
                                        </Dropdown>
                                    )
                                }
                            </div>
                        </div>
                    </Navbar>}
            </div>
        );
    }
}

export default App;