import history from '../history';
import URLS from '../urls';
import axios from 'axios';


export default class Auth {

    user = null;
    authError = null;

    constructor() {
        //this.login = this.login.bind(this);
        this.logout = this.logout.bind(this);
        //this.handleAuthentication = this.handleAuthentication.bind(this);
        this.isAuthenticated = this.isAuthenticated.bind(this);
        this.getAccessToken = this.getAccessToken.bind(this);
        this.getUserInit = this.getUserInit.bind(this);
        this.getUser = this.getUser.bind(this);
        this.userInit = this.userInit.bind(this);
        this.getAuthError = this.getAuthError.bind(this);
        this.setAuthError = this.setAuthError.bind(this);
    }


    userInit(username){
        this.user = {username: username};
    }

    logout(error) {
        let self = this;
        if(error && error !== "valid_logout"){
            this.authError = error;
        }
        let access_token = this.getAccessToken();
        localStorage.removeItem('claimcollector:access_token');
        localStorage.removeItem('claimcollector:expires_at');
        this.user = null;

        let headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': `Bearer ${access_token}`,// + access_token,
            'Access-Control-Allow-Origin': URLS.ssl_app_url,
            'Access-Control-Allow-Credentials': true
        };
        // axios request
        axios({
            url: `${URLS.callable_api_url}/logout`,
            method: 'post',
            headers: headers
        }).then((response) => {
            // User is logged out, navigate to the home route
            if(!error){
                self.authError = null;
            }
            history.replace('/home');
        }).catch((err) => {
            if (err.response.status === 403 && err.response.data.message === "invalid token") {
                // Token has probably expired, backend should have removed it from db. Just redirect
                self.authError = "invalid_token";

                if(error ==="valid_logout"){
                    self.authError = null;
                }

                history.replace('/home');
            }
            //logging(access_token).error("Error occured in getScoreList in App.js", error);
            //defaultLogging().error("Error occured in getScoreList in App.js", error);
        });

    }

    getUserInit(callback) {
        let headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': `Bearer ${this.getAccessToken()}`,// + access_token,
            'Access-Control-Allow-Origin': URLS.ssl_app_url,
            'Access-Control-Allow-Credentials': true
        };
        // axios request
        axios({
            url: `${URLS.callable_api_url}/current-user`,
            method: 'get',
            headers: headers
        }).then((response) => {
            callback(null, response.data);
        }).catch((error) => {
            callback(error);
            //logging(getAccessToken()).error("Error occured in getScoreList in App.js", error);
            //defaultLogging().error("Error occured in getScoreList in App.js", error);
        });
    }

    getAccessToken() {
        const accessToken = localStorage.getItem('claimcollector:access_token');
        if (!accessToken) {
            return null;
        }
        return accessToken;
    }


    getUser(){
        return this.user;
    }


    getAuthError(){
        return this.authError;
    }

    setAuthError(error){
        this.authError = error;
    }

    isAuthenticated() {
        // Check whether the current time is past the
        // access token's expiry time
        let expiresAt = JSON.parse(localStorage.getItem('claimcollector:expires_at'));
        return new Date().getTime() < expiresAt;
    }
}
