import React, { Component } from 'react';
import { Button } from 'react-bootstrap';
import request from 'request';
import URLS from '../urls';
import MuiThemeProvider from 'material-ui/styles/MuiThemeProvider';
import getMuiTheme from 'material-ui/styles/getMuiTheme';
import TextField from 'material-ui/TextField';
import SelectField from 'material-ui/SelectField';
import MenuItem from 'material-ui/MenuItem';


class CreateUser extends Component {

    constructor(props) {
        super(props);
        this.state = {
            pwd: '',
            pwd2: '',
            checked: false,
            age: '',
            gender: '',
            education: '',
            filledAllFields: true,
            pwdErr: '',
            pwd2Err: '',
            copySuccess: '',
            winwidth: 0,
            winheight: 0
        };

        this.inCreateUser = true;

        if(props.location.state){
            this.username = props.location.state.username;
        }

        this.muiTheme = getMuiTheme({
            palette: {
                accent1Color: "#428bca"
            }
        });

        this.muiStyles = {
            underlineStyle: {
                borderColor: "#428bca",
            },
            floatingLabelFocusStyle: {
                color: "#428bca",
            },

            disabledUnderlineStyle: {
                display: 'none'
            },
            disabledFloatingLabelFocusStyle: {
                color: "#000000",
            },
            errorTextStyle: {
                color: "#616161"
            }
        }
    }

    componentWillMount(){
        if(!this.props.location.state){
            //this.props.history.replace(`/${'home'}`);
            this.goTo('home');
        }
        else {
            this.updateDimensions();
        }
    }

    updateDimensions() {
        if(this.inCreateUser) {
            if ((this.state.winheight < 550 && window.innerHeight > 550) || (this.state.winheight > 550 && window.innerHeight < 550)) {
                //console.log("\tRerender createuser.js because of height!, setting heigt to: ", window.innerHeight);
                this.setState({winheight: window.innerHeight});
            }
            if ((this.state.winwidth < 768 && window.innerWidth > 768) || (this.state.winwidth > 768 && window.innerWidth < 768)
                || (this.state.winwidth < 444 && window.innerWidth > 444) || (this.state.winwidth > 444 && window.innerWidth < 444)) {
                //console.log("\tRerender createuser.js because of width!, setting width to: ", window.innerWidth);
                this.setState({winwidth: window.innerWidth});
            }
        }
    }
    componentDidMount(){
        window.addEventListener("resize", this.updateDimensions.bind(this));
    }

    componentWillUnmount(){
        this.inCreateUser = false;
        window.removeEventListener("resize", this.updateDimensions.bind(this));
    }

    goTo(route){
        this.props.history.replace(`/${route}`);
    }

    goToLogin(route, uname) {
        this.props.history.replace({
            pathname:`/${route}`,
            state: {
                username: uname
            }
        });
    }

    disabledCreate(){
        let pwdok = this.state.pwd.length >= 6 && this.state.pwd === this.state.pwd2;
        let ageok = this.state.age !== '';
        let genderok = this.state.gender !== '';
        let eduok = this.state.education !== '';

        return !pwdok || !ageok || !genderok || !eduok || !this.state.checked
    }

    copyToClipboard = (e) => {
        this.textArea.select();
        document.execCommand('copy');
        // This is just personal preference.
        // I prefer to not show the the whole text area selected.
        e.target.focus();
        this.setState({ copySuccess: 'Kopiert!' });
    };

    handlePwdChange(event){
        this.setState({pwd: event.target.value}, function () {
            if(this.state.pwd !== '' && this.state.pwd.length < 6){
                this.setState({pwdErr: 'Passordet må være på minst 6 tegn.'})
            }
            else {
                this.setState({pwdErr: ''})
            }
            if(this.state.pwd2 !== '' && this.state.pwd !== this.state.pwd2){
                this.setState({pwd2Err: 'Passordene er ikke like.'})
            }
            else {
                this.setState({pwd2Err: ''})
            }
        }.bind(this))
    }

    handlePwd2Change(event){
        this.setState({pwd2: event.target.value}, function () {
            if(this.state.pwd2 !== '' && this.state.pwd !== this.state.pwd2){
                this.setState({pwd2Err: 'Passordene er ikke like.'})
            }
            else {
                this.setState({pwd2Err: ''})
            }
        }.bind(this))
    }

    handleCheckboxChange(){
        this.setState({checked: !this.state.checked})
    }

    handleAgeChange = (event, index, value) => this.setState({age: value});

    handleGenderChange = (event, index, value) => this.setState({gender: value});

    handleEduChange = (event, index, value) => this.setState({education: value});

    handleFormSubmit(event){
        let self = this;
        let pwdok = this.state.pwd.length >= 6 && this.state.pwd === this.state.pwd2;
        let ageok = this.state.age !== '';
        let genderok = this.state.age !== '';
        let eduok = this.state.age !== '';

        if(pwdok && ageok && genderok && eduok){
            let headers = {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Access-Control-Allow-Origin': URLS.ssl_app_url,
                'Access-Control-Allow-Credentials': true
            };

            let form = {
                username: self.username,
                password: self.state.pwd,
                age: self.state.age,
                gender: self.state.gender,
                education: self.state.education
            };

            let options = {
                url: `${URLS.callable_api_url}/create-user`,
                method: 'post',
                headers: headers,
                form: form
            };

            request(options, function (err, res, body) {
                //console.log('POST response status: ', res.statusCode);
                //console.log('POST response: ', body);
                if(err){
                    //console.error('error: ', err);
                    //logging(getAccessToken()).error("Error occured in addToApi in Questionnaire.js", err);
                    //defaultLogging().error("Error occured in addToApi in Questionnaire.js", err);
                }else {
                    self.goToLogin('login', self.username);
                }
            })
        }
        else{
            //Ikke korrekt info - feilmld, ikke laget bruker
            this.setState({filledAllFields: false})
        }

        event.preventDefault();
    }

    render() {
        const { isAuthenticated } = this.props.auth;

        let classnameForDiv = "";
        if(window.innerHeight > 550 && window.innerWidth > 768){
            classnameForDiv = "login_div"
        }

        return (

            <div className="container">
                {
                    !isAuthenticated() && (
                        <div className={classnameForDiv}>
                            {this.props.location.state ?
                                <div>
                                    <h2>Lag bruker</h2>
                                    <form onSubmit={this.handleFormSubmit.bind(this)} className="loginForm">
                                        <MuiThemeProvider muiTheme={this.muiTheme}>
                                            <div>
                                                <div className="copydiv">
                                                    <TextField
                                                        floatingLabelText="Brukernavn"
                                                        className="copyfield"
                                                        value={this.username}
                                                        style={{width: 300}}
                                                        underlineFocusStyle={this.muiStyles.disabledUnderlineStyle}
                                                        floatingLabelFocusStyle={this.muiStyles.disabledFloatingLabelFocusStyle}
                                                        inputStyle={this.muiStyles.disabledFloatingLabelFocusStyle}
                                                        errorText={"Vi velger ditt brukernavn for deg, for å sikre full anonymitet."}
                                                        errorStyle={this.muiStyles.errorTextStyle}
                                                        ref={(textarea) => this.textArea = textarea}
                                                    />
                                                    {
                                                        /* Logical shortcut for only displaying the
                                                           button if the copy command exists */
                                                        document.queryCommandSupported('copy') && window.innerWidth > 444 &&
                                                        <div className="copyfield">
                                                            <Button
                                                                bsStyle="default"
                                                                className="btn copy-btn"
                                                                onClick={this.copyToClipboard}
                                                            >
                                                                <i className="fa fa-copy"></i> Kopier
                                                            </Button>
                                                            {"  " + this.state.copySuccess}
                                                        </div>
                                                    }
                                                </div>
                                                {
                                                    /* Logical shortcut for only displaying the
                                                       button if the copy command exists */
                                                    document.queryCommandSupported('copy') &&
                                                    <br/>
                                                }

                                                <TextField
                                                    type="password"
                                                    hintText="Skriv inn ønsket passord"
                                                    underlineFocusStyle={this.muiStyles.underlineStyle}
                                                    floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                                    floatingLabelText="Passord"
                                                    onChange={this.handlePwdChange.bind(this)}
                                                    errorText={this.state.pwdErr}
                                                    style={{width: 300}}
                                                />
                                                <br/>
                                                <TextField
                                                    type="password"
                                                    hintText="Gjenta passordet fra over"
                                                    underlineFocusStyle={this.muiStyles.underlineStyle}
                                                    floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                                    floatingLabelText="Gjenta passord"
                                                    onChange={this.handlePwd2Change.bind(this)}
                                                    errorText={this.state.pwd2Err}
                                                    style={{width: 300}}
                                                />
                                                <br/>

                                                <SelectField
                                                    floatingLabelText="Aldersgruppe"
                                                    value={this.state.age}
                                                    onChange={this.handleAgeChange}
                                                    underlineFocusStyle={this.muiStyles.underlineStyle}
                                                    floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                                    style={{width: 300}}
                                                >
                                                    <MenuItem value={'age18'} primaryText="<18 år"/>
                                                    <MenuItem value={'age19-29'} primaryText="19-29 år"/>
                                                    <MenuItem value={'age30-39'} primaryText="30-39 år"/>
                                                    <MenuItem value={'age40-49'} primaryText="40-49 år"/>
                                                    <MenuItem value={'age50-59'} primaryText="50-59 år"/>
                                                    <MenuItem value={'age60'} primaryText=">60 år"/>
                                                </SelectField>

                                                <br/>

                                                <SelectField
                                                    floatingLabelText="Kjønn"
                                                    value={this.state.gender}
                                                    onChange={this.handleGenderChange}
                                                    underlineFocusStyle={this.muiStyles.underlineStyle}
                                                    floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                                    style={{width: 300}}
                                                >
                                                    <MenuItem value={'kvinne'} primaryText="Kvinne"/>
                                                    <MenuItem value={'mann'} primaryText="Mann"/>
                                                    <MenuItem value={'annet'} primaryText="Annet"/>
                                                </SelectField>

                                                <br/>

                                                <SelectField
                                                    floatingLabelText="Utdanning"
                                                    value={this.state.education}
                                                    onChange={this.handleEduChange}
                                                    underlineFocusStyle={this.muiStyles.underlineStyle}
                                                    floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                                    style={{width: 300}}
                                                >
                                                    <MenuItem value={'humaniora'} primaryText="Humaniora"/>
                                                    <MenuItem value={'realfag'} primaryText="Realfag"/>
                                                    <MenuItem value={'rettsvitenskap'} primaryText="Rettsvitenskap"/>
                                                    <MenuItem value={'medisin'} primaryText="Medisin og Ontologi"/>
                                                    <MenuItem value={'samfunnsvitenskap'}
                                                              primaryText="Samfunnsvitenskap"/>
                                                    <MenuItem value={'kunst'} primaryText="Kunst og Design"/>
                                                    <MenuItem value={'idrett'} primaryText="Idrett"/>
                                                    <MenuItem value={'yrkesfag'} primaryText="Yrkesfag"/>
                                                    <MenuItem value={'annet'} primaryText="Annet"/>
                                                </SelectField>
                                            </div>
                                        </MuiThemeProvider>
                                        <input className="createuser_checkbox" type="checkbox"
                                               onChange={this.handleCheckboxChange.bind(this)}
                                               checked={this.state.checked}/>
                                        Jeg har lest og forstått <a
                                        href="https://claimcollector.idi.ntnu.no/terms-and-conditions" target="_blank">Vilkårene
                                        for
                                        bruk</a> og <a href="https://claimcollector.idi.ntnu.no/privacy"
                                                       target="_blank">Personvernerklæringen</a>.

                                        <br/>

                                        {this.state.filledAllFields ? null :
                                            <p className='wrong'>Du mangler å fylle ut et felt eller flere.</p>
                                        }

                                        <Button
                                            type="submit"
                                            bsStyle="primary"
                                            className="btn login-submit-btn"
                                            disabled={this.disabledCreate()}
                                        >
                                            Lag bruker
                                        </Button>
                                    </form>
                                </div> :
                                null
                            }
                        </div>
                    )
                }
            </div>
        );
    }
}
export default CreateUser;