import React, { Component } from 'react';
import { Button } from 'react-bootstrap';
import request from 'request';
import URLS from '../urls';
import MuiThemeProvider from 'material-ui/styles/MuiThemeProvider';
import TextField from 'material-ui/TextField';

class Login extends Component {

    constructor(props) {
        super(props);
        let uname = '';
        let prevUser = localStorage.getItem('claimcollector:last_user');
        if ( prevUser && prevUser !== 'undefined') {
            uname = prevUser
        }

        if(props.location.state){
            uname = props.location.state.username;
            if (uname) {
                localStorage.setItem('claimcollector:last_user', uname);
            }
        }


        this.state = {
            pwd: '',
            username: uname,
            loginErr: 'false',
            authenticated: false,
            winwidth: 0,
            winheight: 0
        };

        this.inLogin = true;

        this.muiStyles = {
            underlineStyle: {
                borderColor: "#428bca",
            },
            floatingLabelFocusStyle: {
                color: "#428bca",
            }
        }
    }

    componentWillMount(){
        this.updateDimensions();
    }

    updateDimensions() {
        if(this.inLogin) {
            if ((this.state.winheight < 550 && window.innerHeight > 550) || (this.state.winheight > 550 && window.innerHeight < 550)) {
                //console.log("\tRerender login.js because of height!, setting heigt to: ", window.innerHeight);
                this.setState({winheight: window.innerHeight});
            }
            if ((this.state.winwidth < 768 && window.innerWidth > 768) || (this.state.winwidth > 768 && window.innerWidth < 768)) {
                //console.log("\tRerender login.js because of width!, setting width to: ", window.innerWidth);
                this.setState({winwidth: window.innerWidth});
            }
        }
    }
    componentDidMount(){
        window.addEventListener("resize", this.updateDimensions.bind(this));
    }

    componentWillUnmount(){
        this.inLogin = false;
        window.removeEventListener("resize", this.updateDimensions.bind(this));
    }

    goTo(route) {
        this.props.history.replace(`/${route}`)
    }

    disabledLogin(){
        return this.state.username === '' || this.state.pwd === ''
    }

    handlePwdChange(event){
        this.setState({pwd: event.target.value})
    }

    handleUsernameChange(event){
        this.setState({username: event.target.value})
    }

    handleFormSubmit(event){
        let self = this;
        let headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Access-Control-Allow-Origin': URLS.ssl_app_url,
            'Access-Control-Allow-Credentials': true
        };

        let form = {
            username: self.state.username,
            password: self.state.pwd

        };

        let options = {
            url: `${URLS.callable_api_url}/authenticate`,
            method: 'post',
            headers: headers,
            form: form
        };

        request(options, function (err, res, body) {
            //console.log('POST response status: ', res.statusCode);
            //console.log('POST response: ', body);
            if(err){
                self.setState({loginErr: true})
                //logging(getAccessToken()).error("Error occured in addToApi in Questionnaire.js", err);
                //defaultLogging().error("Error occured in addToApi in Questionnaire.js", err);
            }else {
                let body = JSON.parse(res.body);
                if (!body.authenticated) {
                    self.setState({loginErr: true});
                }else {
                    let expires_at = body.expires_at;
                    let token = body.token;
                    self.props.auth.userInit(body.username);
                    localStorage.setItem('claimcollector:access_token', JSON.stringify(token));
                    localStorage.setItem('claimcollector:expires_at', JSON.stringify(expires_at));
                    localStorage.setItem('claimcollector:last_user', body.username);
                    self.goTo('home');
                }


            }
        });
        event.preventDefault();
    }

    render() {
        const { isAuthenticated } = this.props.auth;

        if(isAuthenticated()){
            this.goTo('home');
        }

        let classnameForDiv = "";
        if(window.innerHeight > 550 && window.innerWidth > 768){
            classnameForDiv = "login_div"
        }

        return (

            <div className="container">
                {
                    !isAuthenticated() && (
                        <div className={classnameForDiv}>
                            <h2>Logg inn</h2>
                            <form onSubmit={this.handleFormSubmit.bind(this)} className="loginForm">
                                <MuiThemeProvider>
                                    <div>
                                    <TextField
                                        hintText="Brukernavnet du fikk ved registrering"
                                        underlineFocusStyle={this.muiStyles.underlineStyle}
                                        floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                        floatingLabelText="Brukernavn"
                                        onChange={this.handleUsernameChange.bind(this)}
                                        style={{width: 300}}
                                        value={this.state.username}
                                    />
                                    <br/>
                                    <TextField
                                        type="password"
                                        hintText="Skriv inn passordet ditt"
                                        underlineFocusStyle={this.muiStyles.underlineStyle}
                                        floatingLabelFocusStyle={this.muiStyles.floatingLabelFocusStyle}
                                        floatingLabelText="Passord"
                                        onChange={this.handlePwdChange.bind(this)}
                                        style={{width: 300}}
                                    />
                                    </div>
                                </MuiThemeProvider>

                                <br/>
                                {this.state.loginErr === true ? <p className='wrong'>Feil brukernavn eller passord.</p> : null}

                                <Button
                                    type="submit"
                                    bsStyle="primary"
                                    className="btn login-submit-btn"
                                    disabled={this.disabledLogin()}
                                >
                                    Logg inn
                                </Button>
                            </form>
                        </div>
                    )
                }
            </div>
        );
    }
}
export default Login;