import React, {Component} from 'react';
import { Button } from 'react-bootstrap';
import NotAuthenticated from "../Home/NotAuthenticated";
import axios from 'axios';
import URLS from '../urls';
import {FCW, FNCW, NCW} from "../Constants/AnswerConst";
import Definitions from "./Definitions";
import {logging} from "../Constants/Logging";

class Home extends Component {

    constructor(){
        super();
        this.data = [];
        this.minQ = 10;

        this.user = null;
        this.getAccessToken = null;

        this.state = {
            numQuest: this.minQ,
            inputError: false
        };
    }

    componentDidMount() {
        /*console.log("Home props: ");
        Object.keys(this.props).forEach(key => {
            console.log("\t" + key + " - " +this.props[key]);
        })
        */
    }

    fetchUser(){
        const {getUser} = this.props.auth;

        if(getUser()){
            this.user = getUser();
        }
        else {
            let self = this;
            const {getUserInit} = this.props.auth;
            getUserInit((err, user) => {
                if(user){
                    self.user = user;
                }
                else {
                    self.user = null;
                    const {logout} = self.props.auth;
                    logout();
                    logging(self.getAccessToken).error("Error occured in fetchUser in home.js, no user: ", err);
                    //defaultLogging().error("Error occured in fetchUser in home.js, no user: ", err);
                }
            })
        }
    }

    /*
     Fisher–Yates shuffle algorithm
     */
    shuffle(a) {
        for (let i = a.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [a[i], a[j]] = [a[j], a[i]];
        }
        return a;
    }

    goTo(route){
        const { isAuthenticated } = this.props.auth;

        if (!isAuthenticated()) {
            this.props.auth.logout("invalid_token");
        }
        this.props.history.replace(`/${route}`)
    }

    goToQuestionnaireComponent(route, data, testQ, num_questions) {
        this.props.history.replace({
            pathname:`/${route}`,
            state: {
                data: data,
                testQ: testQ,
                user: this.user,
                minQ: this.minQ,
                num_questions: num_questions
            }
        });
    }

    goToTestQuestionnaire(){

        const { isAuthenticated } = this.props.auth;

        if (!isAuthenticated()) {
            this.props.auth.logout("invalid_token");
        }
        /*
        * 2: FCW = 'Påstand basert på etterprøvbar informasjon, og som er verdt å sjekke'; Eks: 30% av norske barn er fattige.
           1: FNCW = 'Påstand basert på etterprøvbar informasjon, men som ikke er verdt å sjekke'; Eks: Trump ble president i 2016.
           0: NCW = 'Påstand som ikke er basert på etterprøvbar informasjon'; Eks: Jeg liker rømme.
        * */

        this.data = [
            {id: 1, claim: 'Den største veksten i sysselsetting har kommet i privat næringsliv.', answer: '2'},
            {id: 2, claim: 'Nye tall viser økt mistillit til forskere og økt tro på konspirasjonsteorier hos nordmenn.', answer: '2'},
            {id: 3, claim: 'Det blir flere og flere bomstasjoner.', answer: '2'},
            {id: 4, claim: 'Det er ikke lov å flørte lenger.', answer: '1'},
            {id: 5, claim: 'Norge ligger ikke i Europa.', answer: '1'},
            {id: 6, claim: 'Jorda er flat.', answer: '1'},
            {id: 7, claim: 'Vår vurdering er at forslaget gir mer kapasitet i nord.', answer: '0'},
            {id: 8, claim: 'Terror og radikal islamisme står og banker på døren vår.', answer: '0'},
            {id: 9, claim: 'SSBs styring må gjennomgås.', answer: '0'},
            {id: 10, claim: 'Det norske langrennslandslaget er i ferd med å ødelegge gleden ved sporten.', answer: '0'}
        ];

        this.goToQuestionnaireComponent('classification-training', this.shuffle(this.data), true, null);
    }

    handleChange(event) {
        const { isAuthenticated } = this.props.auth;

        if (!isAuthenticated()){
            this.props.auth.logout("invalid_token");
        }
        
        let num_questions = (event.target.validity.valid) ? event.target.value : this.state.numQuest; // can be changed to '' which will blank the field when a non-numeric char is received
        this.setState({numQuest: num_questions});
    }
    
    specificNumberOfQuestionsFromDatabase(event) {

        const { isAuthenticated } = this.props.auth;

        if (!isAuthenticated()) {
            this.props.auth.logout("invalid_token");
        }

        event.preventDefault();
        let num_questions = this.state.numQuest;
        if (num_questions < this.minQ) {
            this.setState({
                inputError: true
            })
        }
        else {
            this.fetchUser();
            let self = this;
            const {getAccessToken} = this.props.auth;
            //console.log('at: ' + access_token);
            let headers = {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${getAccessToken()}`,// + access_token,
                'Access-Control-Allow-Origin': URLS.ssl_app_url,
                'Access-Control-Allow-Credentials': true
            };

            // axios request
            axios({
                url: `${URLS.callable_api_url}/questions/${num_questions}`,
                method: 'get',
                headers: headers
            }).then((response) => {

                let questions = response.data.questions;
                //let baseline = response.data.baseline;

                if(Object.keys(questions).length === 0){
                    this.goTo('thanks');
                }
                else {
                    if (Object.keys(response.data).length === 2) {
                        // We have both baseline and questions
                        // Create new 'ids' for baselines
                        response.data.baseline.map(obj => {
                            obj.id += 'b';
                            return obj;
                        });
                        questions.push(...response.data.baseline);

                        //questions = questions.sort(() => .5 - Math.random());// shuffle
                        questions = self.shuffle(questions);

                        self.data = questions;

                        /*Object.keys(self.data).forEach(key => {
                            console.log("\t" + key + " - " +self.data[key].id);
                        });*/

                        this.goToQuestionnaireComponent('classification', self.data, false, num_questions)
                    }
                    else {
                        //console.log('Response data has only questions');
                        self.data = response.data;
                        this.goToQuestionnaireComponent('classification', self.data, false, num_questions)
                    }
                }


            }).catch((error) => {
                logging(this.getAccessToken).error("Error occured in specificNumberOfQuestionsFromDatabase in home.js", error);
                //defaultLogging().error("Error occured in specificNumberOfQuestionsFromDatabase in home.js", error);
                //console.log('get request error: ', error)
            })

        }
    }

    render() {
        const { isAuthenticated } = this.props.auth;
        if (!isAuthenticated()) {
            this.user = null;
        }

        const {getAuthError} = this.props.auth;

        try{
            const {getAccessToken} = this.props.auth;
            this.getAccessToken = getAccessToken();
        }
        catch (error){
            //console.log("No access token yet");
        }


        return (
            <div className="container">
                {
                    isAuthenticated() && (
                        <div className="home">
                            <h2>Velkommen til ClaimCollector!</h2>
                            <p>
                                Vi trenger hjelp til å klassifisere påstander. En påstand kan høre til en av følgende klasser:
                            </p>
                            <ol>
                                <li> "{FCW}" {/*(eks: {FCWex})*/}</li>
                                <li> "{FNCW()}" {/*(eks: {FNCWex})*/}</li>
                                <li> "{NCW}" {/*(eks: {NCWex})*/}</li>
                            </ol>
                            <p className="topscpre_desc">
                                Det kan til tider være vanskelig å skille disse, og dersom du
                                er usikker på hvilken klasse en påstand hører til, ønsker vi
                                at du velger alternativet "usikker" heller enn å gjette på noen
                                av klassene. I tillegg ber vi deg om at du ser over definisjonene
                                med eksempler som du finner nederst på denne siden,
                                og at du går gjennom en runde eller to med øvingsoppgaver
                                før du begynner på selve klassifiseringen.
                            </p>


                            <p className="topscpre_desc">
                                Klikk for å starte:
                            </p>

                            <Button
                                bsStyle="primary"
                                className="btn"
                                onClick={this.goToTestQuestionnaire.bind(this)}
                            >
                                Øvingsoppgaver
                            </Button>

                            <div className="claimclassification">
                                <form onSubmit={event => this.specificNumberOfQuestionsFromDatabase(event)}>
                                    <p>Hvor mange påstander vil du klassifisere i denne runden?
                                        (minimum er <b>{this.minQ}</b>)
                                        {this.state.inputError ? <p style={{color: 'red'}}>Du må ha minst {this.minQ} spørsmål.</p>: null}
                                    </p>

                                    <p className="inline_p">
                                        <input type="text" pattern="[0-9]*" maxLength="4" name="questions" value={this.state.numQuest}
                                               onChange={this.handleChange.bind(this)}/>
                                    </p>
                                    <Button
                                        type="submit"
                                        bsStyle="primary"
                                        className="btn"
                                    >
                                        Påstandsklassifisering
                                    </Button>
                                </form>
                            </div>
                            <br/>
                            <p className="topscpre_desc">
                                Etter at du har fullført en runde med klassifisering kan du se resultatene dine og andres ved å
                                trykke på knappen "Resultater" i menyen øverst. Som takk for hjelpen ønsker vi å
                                dele ut et gavekort fra Komplett til en tilfeldig person blandt de som har klassifisert minst 20 påstander.
                                Trekningen vil bli gjennomført ved avslutning av innsamlingen (15. april). Se mer info om trekningen på
                                resultatsiden når du har klassifisert minst 20 påstander.
                            </p>
                            <Definitions {...this.props} collapsable={false}/>
                        </div>
                    )
                }
                {
                    !isAuthenticated() && (
                        <div>
                            <div className="home">
                                <h2>Velkommen til ClaimCollector!</h2>
                                <p>
                                    Denne nettsiden er opprettet for å samle data til en masteroppgave som skal
                                    plukke ut påstander fra norske nyheter som er verdt å se nærmere på (f.eks. sant/usant).
                                </p>
                                <p>
                                    Innsamlingen vil foregå til og med <strong>15. april</strong>. Etter dette er det ikke sikkert at
                                    nye data vil bli inkludert i oppgaven.
                                </p>
                                <p>
                                    Til dette trenger vi hjelp, og ber om at du logger på og klassifiserer noen
                                    påstander som du får oppgitt. Dette kan senere bli brukt til å trene opp programmer
                                    som kan gjenkjenne aktuelle påstander.
                                </p>
                                <p>Vi ønsker i tillegg å se på hvordan forskjellige grupper svarer.</p>
                            </div>

                            <NotAuthenticated {...this.props} authError={getAuthError()}/>
                        </div>
                    )
                }
            </div>
        );
    }
}

export default Home;
