import React, { Component } from 'react';
import NotAuthenticated from "./NotAuthenticated";
import { BootstrapTable, TableHeaderColumn } from 'react-bootstrap-table';
import { Button } from 'react-bootstrap';
import URLS from "../urls";
import axios from "axios/index";
import {logging} from "../Constants/Logging";

class Topscore extends Component {

    constructor(props){
        super(props);
        this.state = {
            distCode: ''
        };

        this.disableGenerate = false;
        this.showingUserlist = null;
        this.userInfo = null;

        if(props.location.state) {
            this.showingUserlist = props.location.state.scores.scores;
            this.userInfo = this.getInfoFromUser(props.location.state.user.username, this.showingUserlist);
        }
    }

    goTo(route){
        this.props.history.replace(`/${route}`);
    }

    componentWillMount(){
        if(!this.props.location.state){
            //this.props.history.replace(`/${'home'}`);
            this.goTo('home');
        }
    }

    customSortFunc(a, b, order) {   // order is desc or asc
        let a_score = parseFloat(a.score);
        let b_score = parseFloat(b.score);
        if (order === 'asc') {
            return a_score - b_score;
        } else {
            return b_score - a_score;
        }
    }

    getInfoFromUser(nickname, userlist){
        let result = null;
        Object.keys(userlist).forEach(key => {
            if(userlist[key].username === nickname){
                result = [userlist[key].score, userlist[key].answer_count, userlist[key].code_distributed];
            }
        });
        return result;
    }

    compare_users(obj1, obj2) {
        let score_o1 = parseFloat(obj1.score);
        let score_o2 = parseFloat(obj2.score);
        if (score_o1 === score_o2){
            if (obj1.answer_count > obj2.answer_count) {
                return -1
            }
            else if (obj1.answer_count < obj2.answer_count) {
                return 1
            }
            else {
                return 0
            }
        }
        else if (score_o1 > score_o2) {
            return -1
        }
        else {
            return 1
        }
    }

    getCode(){
        let self = this;
        const {getAccessToken} = this.props.auth;
        let headers = {
            'Content-Type': 'application/json',
            'Authorization': `Bearer ${getAccessToken()}`,// + access_token,
            'Access-Control-Allow-Origin': URLS.ssl_app_url,
            'Access-Control-Allow-Credentials': true
        };
        // axios request
        axios({
            url: `${URLS.callable_api_url}/generate-code`,
            method: 'get',
            headers: headers
        }).then((response) => {
            let code = response.data.distcode;
            this.disableGenerate = true;
            self.setState({distCode: code});
        }).catch((error) => {
            //console.error('error: ', error);
            logging(getAccessToken()).error("Error occurred in getCode in Topscore.js", error);
            //defaultLogging().error("Error occurred in getScoreList in App.js", error);
        })
    }

    render() {
        const { isAuthenticated } = this.props.auth;

        let options = {
            defaultSortName: 'placement',
            defaultSortOrder: 'asc',
            noDataText: 'Ingen resultater å vise'
        };

        if(this.showingUserlist) {

            this.showingUserlist.sort(this.compare_users); // sort with compareTo logic

            let prev = this.showingUserlist[0];
            let counter = 1;

            this.showingUserlist.map(obj => {
                if (this.compare_users(prev, obj) === 0) {
                    obj.placement = counter;
                } else {
                    obj.placement = counter + 1;
                    counter++;
                }
                prev = obj;
                return prev;
            });
        }

        return (

            <div className="container">
                {
                    isAuthenticated() && (
                        <div className="topscore">
                            <h2>Resultater: </h2>

                            {this.userInfo ?
                                <div>
                                    <p className="usersScore">
                                        Dine resultater per nå: {this.userInfo[0]} (treffsikkerhet), {this.userInfo[1]} (påstander klassifisert).
                                    </p>
                                    {this.userInfo[1] >= 20 ?
                                        <div>
                                            {this.userInfo[2] === 0 ?
                                                <div>
                                                    <p>Du har nå klassifisert nok påstander og kan delta i trekningen med din deltakerkode som du får ved å trykke på knappen under.
                                                    </p>
                                                    <p>
                                                        <strong>VIKTIG!</strong> Du vil kun få tildelt koden din én gang, og den vil forsvinne dersom du går vekk
                                                        fra siden du ser nå. Dette betyr at dersom du ikke benytter deg
                                                        av den før den forsvinner vil du dessverre ikke kunne delta i trekningen. Dette er gjort
                                                        for å holde alle brukere anonyme på denne nettsiden.
                                                    </p>
                                                    <Button
                                                        bsStyle="primary"
                                                        className="btn"
                                                        onClick={this.getCode.bind(this)}
                                                        disabled={this.disableGenerate}
                                                    >
                                                        Generer kode!
                                                    </Button>
                                                    {this.state.distCode !== '' ?
                                                        <div className="inline_div_marg">
                                                            {this.state.distCode === undefined ?
                                                                <p className="inline_p">Du skal allerede ha fått utdelt din kode.</p>:
                                                                <p className="inline_p">Din kode er: <strong>{this.state.distCode}</strong>. Skriv inn den og din epost
                                                                    på <a href="https://goo.gl/forms/nAFLdAifQrDo0x5K2" target="_blank">https://goo.gl/forms/nAFLdAifQrDo0x5K2</a> for å delta i trekningen.
                                                                </p>
                                                            }
                                                        </div>
                                                        :
                                                        null}
                                                </div>
                                                :
                                                <p><strong>Din status for trekingen:</strong> Du har allerede fått utdelt din kode.</p>
                                            }
                                        </div>
                                        : <p>Du har for øyeblikket ikke klassifisert nok påstander til å delta i trekningen.</p>
                                    }
                                </div>
                                :
                                <p>
                                    Du har ikke klassifisert noen påstander enda, og har dermed ingen resultater å vise.
                                </p>
                            }


                            <h3 className="topscore_h">Toppscore-liste: </h3>

                            <div className={'topscore_explain'}>
                                <p><b>Plassering:</b> basert på treffsikkerhet, og så på antall påstander klassifisert for de som har lik treffsikkerhet.</p>
                                <p><b>Treffsikkerhet:</b> estimat av treffsikkerhet på svarene.</p>
                                <p><b>Påstander klassifisert:</b> antallet påstander klassifisert (ikke trykket "Usikker" på).</p>
                            </div>

                            <BootstrapTable data={this.showingUserlist} options={options} striped={true} hover={true}>
                                <TableHeaderColumn dataField="placement" dataSort={true}>Plassering</TableHeaderColumn>
                                <TableHeaderColumn dataField="username" isKey={true} dataSort={true}>Navn</TableHeaderColumn>
                                <TableHeaderColumn dataField="score" dataSort sortFunc={this.customSortFunc}>Treffsikkerhet</TableHeaderColumn>
                                <TableHeaderColumn dataField="answer_count" dataSort={true}>Påstander klassifisert</TableHeaderColumn>
                            </BootstrapTable>

                        </div>
                    )
                }
                {
                    !isAuthenticated() && (
                        <NotAuthenticated {...this.props}/>
                    )
                }
            </div>


        );
    }
}
export default Topscore;