import React, {Component} from "react";
import Question from './Question';
import { Button } from 'react-bootstrap';
import ControlBar from './ControlBar';
import request from 'request';
import URLS from '../urls';
import Definitions from "../Home/Definitions";
import {logging} from "../Constants/Logging";

class Questionnaire extends Component {

    constructor(props){
        super(props);
        this.state = {
            currentClaim: 0,
            nextBtnDisabled: true
        };

        this.testQ = null;
        this.user = null;
        this.data = null;
        this.answer_data = {};
        this.minQ = null;
        this.qaDict = {};
        this.num_questions = null;

        this.goToClaim = this.goToClaim.bind(this);
        this.showNextClaim = this.showNextClaim.bind(this);
        this.showPrevClaim = this.showPrevClaim.bind(this);
        this.questionnaireEnableNextButton = this.questionnaireEnableNextButton.bind(this);
        this.getQuestionAnswers = this.getQuestionAnswers.bind(this);
        this.getQuestionAnswerWithIxDict = this.getQuestionAnswerWithIxDict.bind(this);
        this.getSubmitBtn = this.getSubmitBtn.bind(this);
    }

    goTo(route){
        this.props.history.replace(`/${route}`);
    }

    componentWillMount(){
        if(!this.props.location.state){
            //this.props.history.replace(`/${'home'}`);
            this.goTo('home');
        }
    }

    onUnload(event) { // the method that will be used for both add and remove event
        const confirmationMessage = 'Denne siden ber deg bekrefte at du ønsker å forlate siden - data som du allerede har skrevet inn vil ikke lagres.';
        event.returnValue = confirmationMessage;     // Gecko, Trident, Chrome 34+
        return confirmationMessage;
    }

    componentDidMount(){
        if (this.props.location.state){
            if (!this.props.location.state.testQ){
                window.addEventListener("beforeunload", this.onUnload)
            }
        }
        //console.log('User in questionnaire 1: ', this.user);
    }

    componentWillUnmount() {
        if (this.props.location.state){
            if (!this.props.location.state.testQ){
                window.removeEventListener("beforeunload", this.onUnload)
            }
        }
    }

    //Transform ids from '33b' to 33
    normalizeBaselineIds(questions){
        return questions.map(obj => {
            obj.id = parseInt(obj.id, 10);
            return obj;
        });
    }

    separateQuestionData(data){
        let questions = [];
        let baseline = [];
        data.map(obj => {
            //console.log('separate obj: ', obj);
            obj.user = this.user;
            if (typeof obj.id === 'string'){
                baseline.push(obj)
            }else {
                questions.push(obj)
            }
            return obj;
        });
        baseline = this.normalizeBaselineIds(baseline);
        return {questions: questions, baseline: baseline}
    }

    addToApi() {
        //console.log('Trying POST request...');
        let user = this.user;
        // console.log('User in addAPiTest: ', user);
        const {getAccessToken} = this.props.auth;

        if (user === null || user === undefined || user === 'undefined') {
            //console.error('ERROR: No user defined: ', user);
            logging(getAccessToken()).error("Error occured in addToApi in Questionnaire.js: 'No user defined: '", user);
            //defaultLogging().error("Error occured in addToApi in Questionnaire.js: 'No user defined: '", user);
        }
        else {
            let props_data = this.data;

            // Set answer to the specific answer form radio buttons
            props_data.map(obj => {
                Object.keys(this.answer_data).forEach(key => {
                    if(parseInt(obj.id, 10) === parseInt(key, 10)) {
                        obj.answer = this.answer_data[key];
                    }
                });
                return obj;
            });

            // Filter out questions without answers
            let filtered_props_data = props_data.filter(obj => {
                return obj.answer !== undefined;
            });
            //console.log('Filtered data: ', filtered_props_data);

            let data = this.separateQuestionData(filtered_props_data);
            //console.log('final input data: ', data);

            //console.log('at: ' + access_token);
            let headers = {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${getAccessToken()}`,// + access_token,
                'Access-Control-Allow-Origin': URLS.ssl_app_url,
                'Access-Control-Allow-Credentials': true
            };
            /*let data = {
                questions: questions,

            };*/
            /*
             Axios wont send the data with the request, research this issue if time later. normal request works.
             */

            // axios request
            /*axios({
             url: 'http://localhost:8000/questions/done',
             method: 'post',
             headers: headers,
             data: data
             }).then((response) => {
             console.log('POST response status: ', response.status)
             }).catch((error) => {
             console.log('POST AXIOS Error: ', error)
             })*/


            let options = {
                url: `${URLS.callable_api_url}/questions/done`,
                method: 'post',
                headers: headers,
                form: data
            };

            request(options, function (err, res, body) {
                //console.log('POST response status: ', res.statusCode);
                //console.log('POST response: ', body);
                if(err){
                    logging(getAccessToken()).error("Error occured in addToApi in Questionnaire.js", err);
                    //defaultLogging().error("Error occured in addToApi in Questionnaire.js", err);
                }
            })
        }

    }

    handleFormSubmit (formSubmitEvent) {
        formSubmitEvent.preventDefault();

        const { isAuthenticated } = this.props.auth;

        if (!isAuthenticated()) {
            this.props.auth.logout("invalid_token");
        }
        else {

            if (this.testQ) {
                this.goTo('home');
            }
            else {
                if (window.confirm("Du har klassifisert " + Object.keys(this.answer_data).length + " av " + this.data.length +
                        " påstander i denne runden.\nVed å trykke OK sender du inn svarene på påstandene du har klassifisert.")) {
                    this.addToApi();
                    this.goTo('done');
                }
            }
        }
    }

    showNextClaim(){
        let nextClaim = this.getNextClaim(this.state.currentClaim, this.data.length);
        this.goToClaim(nextClaim);
        this.setState({
            nextBtnDisabled: true
        });
        this.questionnaireEnableNextButton(nextClaim);
    }

    getNextClaim(claimnr, length){
        let nextClaim = claimnr +1;
        if(nextClaim >= length){
            nextClaim = 0;
        }
        return nextClaim;
    }

    showPrevClaim(){
        let prevClaim = this.getPrevClaim(this.state.currentClaim, this.data.length);
        this.goToClaim(prevClaim);
        this.questionnaireEnableNextButton(prevClaim);
    }

    getPrevClaim(claimnr, length){
        let prevClaim = claimnr -1;
        if(prevClaim <0){
            prevClaim = length - 1;
        }
        return prevClaim;
    }

    goToClaim(index){
        this.setState({
            currentClaim: index
        });
    }

    getQuestionAnswers(key, answer){
        this.answer_data[key] = answer;
    }

    getQuestionAnswerWithIxDict(index, answer, correctAnswer){
        this.qaDict[index] = [answer, correctAnswer];
    }

    questionnaireEnableNextButton(currentClaim){
        this.setState({
            nextBtnDisabled: !(currentClaim in this.qaDict)
        });
    }

    getQuestions(){
        return this.data.map((obj, index) => { //obj contains: id, question, answer
            const isCurrentSlide = index === this.state.currentClaim;
            const questionClass = (isCurrentSlide ? "is-current" : "not-current");

            return  <Question
                key={obj.id}
                keyToUse={obj.id}
                index={index}
                {...this.props}
                claim={obj.claim}
                class={questionClass}
                answer={obj.answer}
                testQ={this.testQ}
                questionnaireEnableNextButton={this.questionnaireEnableNextButton}
                getQuestionAnswers={this.getQuestionAnswers}
                getQuestionAnswerWithIxDict={this.getQuestionAnswerWithIxDict}
            />
        });
    }

    enableDisableSubmitBtn(){
        if (this.data.length >0 && this.data.length < this.minQ) {
            return (Object.keys(this.answer_data).length < this.data.length);
        }

        if(this.testQ){
            return false;
        }
        else{
            return Object.keys(this.answer_data).length < this.minQ;
        }
    }

    getSubmitBtn(){
        if(this.testQ){
            return (<Button
                type="submit"
                bsStyle="primary"
                className="btn submit-btn"
                disabled={this.enableDisableSubmitBtn()}
            >
                Avslutt
            </Button>)
        }
        else{
            return (
                <Button
                    type="submit"
                    bsStyle="primary"
                    className="btn submit-btn"
                    disabled={this.enableDisableSubmitBtn()}
                >
                    Fullfør
                </Button>
            )

        }

    }

    render(){
        const { isAuthenticated } = this.props.auth;

        if (!isAuthenticated()){
            this.props.auth.logout("invalid_token");
        }

        if(this.props.location.state){
            this.testQ = this.props.location.state.testQ;
            this.user = this.props.location.state.user;
            this.data = this.props.location.state.data;
            this.minQ = this.props.location.state.minQ;
            this.num_questions = this.props.location.state.num_questions;
        }

        return(
            <div className="container">
                {
                    isAuthenticated() && (
                        <div className="questionnaire">
                            {this.props.location.state ?
                                <div>
                                    <form onSubmit={this.handleFormSubmit.bind(this)} className="Qform">

                                        <p className="progressBar">Påstand {this.state.currentClaim + 1} av {this.data.length}</p>
                                        {(this.data.length >0 && this.data.length < this.num_questions) ?
                                            <p><i className="fa fa-exclamation-triangle" aria-hidden="true"></i> Det er ikke flere
                                                enn {this.data.length} påstander igjen som trenger å bli
                                                klassifisert av deg. Du får dermed kun se disse.</p>
                                            : null}

                                        <ol>
                                            {this.getQuestions()}
                                        </ol>
                                        <div className="questionnaire-info">
                                            {(this.state.nextBtnDisabled && this.state.currentClaim === 0) ?
                                                <p>"Neste"-knappen
                                                    vil bli mulig å trykke på når du har valgt et av
                                                    alternativene.</p>
                                                : null}

                                            {(this.data.length >0 && this.data.length < this.minQ && !this.testQ) ?
                                                (Object.keys(this.answer_data).length < this.data.length ?
                                                    <p>"Fullfør"-knappen
                                                        vil være mulig å trykke på når du har klassifisert
                                                        minst {this.minQ} påstander, eller når du nærmer deg antall spørsmål i databasen.</p>
                                                    : null) :
                                                ((Object.keys(this.answer_data).length < this.minQ && !this.testQ) ?
                                                    <p>"Fullfør"-knappen
                                                        vil være mulig å trykke på når du har klassifisert
                                                        minst {this.minQ} påstander, eller de resterende påstandene om dette er færre.</p>
                                                    : null)
                                            }
                                        </div>
                                        <ControlBar
                                            showPrevClaim={this.showPrevClaim}
                                            showNextClaim={this.showNextClaim}
                                            currentClaim={this.state.currentClaim}
                                            lastClaim={this.data.length - 1}
                                            nextBtnDisabled={this.state.nextBtnDisabled}
                                            getSubmitBtn={this.getSubmitBtn}
                                        />
                                    </form>
                                    {this.testQ ?
                                        // If this is a testQuestionnaire
                                        (this.qaDict[this.state.currentClaim] && (this.qaDict[this.state.currentClaim][0] !== this.qaDict[this.state.currentClaim][1]) ?
                                        // If the claim has an answer, and if the claim do not have the correct answer
                                            <Definitions {...this.props} collapsable={true} detailed={true}/> // Show detailed definitions
                                            : <Definitions {...this.props} collapsable={true} detailed={false}/>)
                                        :
                                        <Definitions {...this.props} collapsable={true} detailed={false}/>
                                    }
                                </div>:
                                null}
                        </div>
                    )
                }
            </div>
        )
    }
}

export default Questionnaire;